@@ -11,7 +11,7 @@ Maintainer: U{Itamar Shtull-Trauring<mailto:twisted@itamarst.org>}
 """
 
 import socket # needed only for sync-dns
-from zope.interface import implements
+from zope.interface import implements, classImplements
 
 import imp
 import sys
@@ -29,7 +29,7 @@ from twisted.internet.interfaces import IReactorCore, IReactorTime, IReactorThre
 from twisted.internet.interfaces import IResolverSimple, IReactorPluggableResolver
 from twisted.internet.interfaces import IConnector, IDelayedCall
 from twisted.internet import main, error, abstract, defer, threads
-from twisted.python import log, failure, reflect, components
+from twisted.python import log, failure, reflect
 from twisted.python.runtime import seconds, platform
 from twisted.internet.defer import Deferred, DeferredList
 from twisted.persisted import styles
@@ -46,10 +46,27 @@ class DelayedCall(styles.Ephemeral):
     debug = False
     _str = None
 
-    def __init__(self, time, func, args, kw, cancel, reset):
+    def __init__(self, time, func, args, kw, cancel, reset, seconds=None):
+        """
+        @param time: Seconds from the epoch at which to call C{func}.
+        @param func: The callable to call.
+        @param args: The positional arguments to pass to the callable.
+        @param kw: The keyword arguments to pass to the callable.
+        @param cancel: A callable which will be called with this
+            DelayedCall before cancellation.
+        @param reset: A callable which will be called with this
+            DelayedCall after changing this DelayedCall's scheduled
+            execution time. The callable should adjust any necessary
+            scheduling details to ensure this DelayedCall is invoked
+            at the new appropriate time.
+        @param seconds: If provided, a no-argument callable which will be
+            used to determine the current time any time that information is
+            needed.
+        """
         self.time, self.func, self.args, self.kw = time, func, args, kw
         self.resetter = reset
         self.canceller = cancel
+        self.seconds = seconds
         self.cancelled = self.called = 0
         self.delayed_time = 0
         if self.debug:
@@ -98,7 +115,10 @@ class DelayedCall(styles.Ephemeral):
         elif self.called:
             raise error.AlreadyCalled
         else:
-            new_time = seconds() + secondsFromNow
+            if self.seconds is None:
+                new_time = seconds() + secondsFromNow
+            else:
+                new_time = self.seconds() + secondsFromNow
             if new_time < self.time:
                 self.delayed_time = 0
                 self.time = new_time
@@ -155,8 +175,12 @@ class DelayedCall(styles.Ephemeral):
         else:
             func = None
 
+        if self.seconds is None:
+            now = seconds()
+        else:
+            now = self.seconds()
         L = ["<DelayedCall %s [%ss] called=%s cancelled=%s" % (
-                id(self), self.time - seconds(), self.called, self.cancelled)]
+                id(self), self.time - now, self.called, self.cancelled)]
         if func is not None:
             L.extend((" ", func, "("))
             if self.args:
@@ -174,8 +198,6 @@ class DelayedCall(styles.Ephemeral):
         return "".join(L)
 
 
-components.backwardsCompatImplements(DelayedCall)
-
 class ThreadedResolver:
     implements(IResolverSimple)
 
@@ -219,8 +241,6 @@ class ThreadedResolver:
         lookupDeferred.addBoth(self._checkTimeout, name, lookupDeferred)
         return userDeferred
 
-components.backwardsCompatImplements(ThreadedResolver)
-
 class BlockingResolver:
     implements(IResolverSimple)
 
@@ -231,20 +251,14 @@ class BlockingResolver:
             msg = "address %r not found" % (name,)
             err = error.DNSLookupError(msg)
             return defer.fail(err)
-        else:      
+        else:
             return defer.succeed(address)
 
-components.backwardsCompatImplements(BlockingResolver)
-
-class ReactorBase:
+class ReactorBase(object):
     """Default base class for Reactors.
     """
 
-    _implList = [IReactorCore, IReactorTime, IReactorPluggableResolver]
-    if platform.supportsThreads():
-        _implList.append(IReactorThreads)
-    implements(*_implList)
-    del _implList
+    implements(IReactorCore, IReactorTime, IReactorPluggableResolver)
 
     installed = 0
     usingThreads = 0
@@ -288,7 +302,7 @@ class ReactorBase:
             # if the waker isn't installed, the reactor isn't running, and
             # therefore doesn't need to be woken up
 
-    def doIteration(self):
+    def doIteration(self, delay):
         """Do one iteration over the readers and writers we know about."""
         raise NotImplementedError
 
@@ -403,7 +417,7 @@ class ReactorBase:
         # now that we've called all callbacks, no need to store
         # references to them anymore, in fact this can cause problems.
         del self._eventTriggers[eventType]
-    
+
     def addSystemEventTrigger(self, _phase, _eventType, _f, *args, **kw):
         """See twisted.internet.interfaces.IReactorCore.addSystemEventTrigger.
         """
@@ -468,7 +482,7 @@ class ReactorBase:
         except ValueError:
             # element was not found in heap - oh well...
             pass
-        
+
     def _cancelCallLater(self, tple):
         self._cancellations+=1
 
@@ -495,16 +509,16 @@ class ReactorBase:
                 call.activate_delay()
                 heappush(self._pendingTimedCalls, call)
         self._newTimedCalls = []
-        
+
     def timeout(self):
         # insert new delayed calls to make sure to include them in timeout value
         self._insertNewDelayedCalls()
-        
+
         if not self._pendingTimedCalls:
             return None
-        
+
         return max(0, self._pendingTimedCalls[0].time - seconds())
-        
+
     def runUntilCurrent(self):
         """Run all pending timed calls.
         """
@@ -513,24 +527,30 @@ class ReactorBase:
             # making them, in case another call is added to the queue
             # while we're in this loop.
             count = 0
+            total = len(self.threadCallQueue)
             for (f, a, kw) in self.threadCallQueue:
                 try:
                     f(*a, **kw)
                 except:
                     log.err()
                 count += 1
+                if count == total:
+                    break
             del self.threadCallQueue[:count]
-        
+            if self.threadCallQueue:
+                if self.waker:
+                    self.waker.wakeUp()
+ 
         # insert new delayed calls now
         self._insertNewDelayedCalls()
-        
+
         now = seconds()
         while self._pendingTimedCalls and (self._pendingTimedCalls[0].time <= now):
             call = heappop(self._pendingTimedCalls)
             if call.cancelled:
                 self._cancellations-=1
                 continue
-            
+
             if call.delayed_time > 0:
                 call.activate_delay()
                 heappush(self._pendingTimedCalls, call)
@@ -550,7 +570,7 @@ class ReactorBase:
                     e += "\n"
                     log.msg(e)
 
-        
+
         if (self._cancellations > 50 and
              self._cancellations > len(self._pendingTimedCalls) >> 1):
             self._cancellations = 0
@@ -580,13 +600,13 @@ class ReactorBase:
         def _initThreadPool(self):
             from twisted.python import threadpool
             self.threadpool = threadpool.ThreadPool(0, 10, 'twisted.internet.reactor')
-            self.threadpool.start()
+            self.callWhenRunning(self.threadpool.start)
             self.addSystemEventTrigger('during', 'shutdown', self.threadpool.stop)
 
         def callInThread(self, _callable, *args, **kwargs):
             """See twisted.internet.interfaces.IReactorThreads.callInThread.
             """
-            if not self.threadpool:
+            if self.threadpool is None:
                 self._initThreadPool()
             self.threadpool.callInThread(_callable, *args, **kwargs)
 
@@ -605,7 +625,8 @@ class ReactorBase:
             # See comment in the other callFromThread implementation.
             self.threadCallQueue.append((f, args, kw))
 
-components.backwardsCompatImplements(ReactorBase)
+if platform.supportsThreads():
+    classImplements(ReactorBase, IReactorThreads)
 
 
 class BaseConnector(styles.Ephemeral):
@@ -689,12 +710,10 @@ class BaseConnector(styles.Ephemeral):
     def getDestination(self):
         raise NotImplementedError, "implement in subclasses"
 
-components.backwardsCompatImplements(BaseConnector)
-
 
 class BasePort(abstract.FileDescriptor):
     """Basic implementation of a ListeningPort.
-    
+
     Note: This does not actually implement IListeningPort.
     """
 
