@@ -15,20 +15,20 @@ import rfc822
 from twisted.trial import unittest
 import tempfile
 
-from zope.interface import providedBy
+from zope.interface import providedBy, Interface, implements
 
-from twisted.trial import unittest, util as tutil
+from twisted.trial import unittest
 from twisted.mail import smtp
 from twisted.mail import pop3
 from twisted.names import dns
 from twisted.protocols import basic
 from twisted.internet import protocol
 from twisted.internet import defer
+from twisted.internet.defer import Deferred
 from twisted.internet import reactor
 from twisted.internet import interfaces
 from twisted.internet.error import DNSLookupError, CannotListenError
 from twisted.internet import address
-from twisted.python import components
 from twisted.python import failure
 from twisted.python import util
 
@@ -40,6 +40,9 @@ import twisted.mail.relaymanager
 import twisted.mail.protocols
 import twisted.mail.alias
 
+from twisted.names.error import DNSNameError
+from twisted.names.dns import Record_MX
+
 from twisted import cred
 import twisted.cred.credentials
 import twisted.cred.checkers
@@ -221,29 +224,42 @@ class MaildirAppendStringTestCase(unittest.TestCase):
     def tearDown(self):
         shutil.rmtree(self.d)
 
+    def _append(self, ignored, mbox):
+        d = mbox.appendMessage('TEST')
+        return self.assertFailure(d, Exception)
+
+    def _setState(self, ignored, mbox, rename=None, write=None, open=None):
+        if rename is not None:
+            mbox.AppendFactory._renameState = rename
+        if write is not None:
+            mbox.AppendFactory._writeState = write
+        if open is not None:
+            mbox.AppendFactory._openstate = open
+
     def testAppend(self):
         mbox = mail.maildir.MaildirMailbox(self.d)
         mbox.AppendFactory = FailingMaildirMailboxAppendMessageTask
+        ds = []
         for i in xrange(1, 11):
-            self.assertEquals(
-                unittest.wait(mbox.appendMessage("X" * i)),
-                None)
+            ds.append(mbox.appendMessage("X" * i))
+            ds[-1].addCallback(self.assertEqual, None)
+        d = defer.gatherResults(ds)
+        d.addCallback(self._cbTestAppend, mbox)
+        return d
+
+    def _cbTestAppend(self, result, mbox):
         self.assertEquals(len(mbox.listMessages()),
                           10)
         self.assertEquals(len(mbox.getMessage(5).read()), 6)
         # test in the right order: last to first error location.
         mbox.AppendFactory._renamestate = False
-        self.failUnless(isinstance(unittest.deferredError(mbox.appendMessage("TEST")),
-                                                        failure.Failure))
-        mbox.AppendFactory._renamestate = True
-        mbox.AppendFactory._writestate = False
-        self.failUnless(isinstance(unittest.deferredError(mbox.appendMessage("TEST")),
-                                                        failure.Failure))
-        mbox.AppendFactory._writestate = True
-        mbox.AppendFactory._openstate = False
-        self.failUnless(isinstance(unittest.deferredError(mbox.appendMessage("TEST")),
-                                                        failure.Failure))
-        mbox.AppendFactory._openstate = True
+        d = self._append(None, mbox)
+        d.addCallback(self._setState, mbox, rename=True, write=False)
+        d.addCallback(self._append, mbox)
+        d.addCallback(self._setState, mbox, write=True, open=False)
+        d.addCallback(self._append, mbox)
+        d.addCallback(self._setState, mbox, open=True)
+        return d
 
 
 class MaildirAppendFileTestCase(unittest.TestCase):
@@ -256,14 +272,19 @@ class MaildirAppendFileTestCase(unittest.TestCase):
 
     def testAppend(self):
         mbox = mail.maildir.MaildirMailbox(self.d)
+        ds = []
+        def _check(res, t):
+            t.close()
+            self.assertEqual(res, None)
         for i in xrange(1, 11):
             temp = tempfile.TemporaryFile()
             temp.write("X" * i)
             temp.seek(0,0)
-            self.assertEquals(
-                unittest.wait(mbox.appendMessage(temp)),
-                None)
-            temp.close()
+            ds.append(mbox.appendMessage(temp))
+            ds[-1].addCallback(_check, temp)
+        return defer.gatherResults(ds).addCallback(self._cbTestAppend, mbox)
+
+    def _cbTestAppend(self, result, mbox):
         self.assertEquals(len(mbox.listMessages()),
                           10)
         self.assertEquals(len(mbox.getMessage(5).read()), 6)
@@ -362,7 +383,7 @@ class MaildirDirdbmDomainTestCase(unittest.TestCase):
         self.failUnless(cred.credentials.IUsernamePassword in creds[0].credentialInterfaces)
 
     def testRequestAvatar(self):
-        class ISomething(components.Interface):
+        class ISomething(Interface):
             pass
 
         self.D.addUser('user', 'password')
@@ -391,6 +412,46 @@ class MaildirDirdbmDomainTestCase(unittest.TestCase):
         creds = cred.credentials.UsernamePassword('user', 'password')
         self.assertEquals(database.requestAvatarId(creds), 'user')
 
+
+class StubAliasableDomain(object):
+    """
+    Minimal testable implementation of IAliasableDomain.
+    """
+    implements(mail.mail.IAliasableDomain)
+
+    def exists(self, user):
+        """
+        No test coverage for invocations of this method on domain objects,
+        so we just won't implement it.
+        """
+        raise NotImplementedError()
+
+
+    def addUser(self, user, password):
+        """
+        No test coverage for invocations of this method on domain objects,
+        so we just won't implement it.
+        """
+        raise NotImplementedError()
+
+
+    def getCredentialsCheckers(self):
+        """
+        This needs to succeed in order for other tests to complete
+        successfully, but we don't actually assert anything about its
+        behavior.  Return an empty list.  Sometime later we should return
+        something else and assert that a portal got set up properly.
+        """
+        return []
+
+
+    def setAliasGroup(self, aliases):
+        """
+        Just record the value so the test can check it later.
+        """
+        self.aliasGroup = aliases
+
+
 class ServiceDomainTestCase(unittest.TestCase):
     def setUp(self):
         self.S = mail.mail.MailService()
@@ -402,11 +463,24 @@ class ServiceDomainTestCase(unittest.TestCase):
         self.tmpdir = self.mktemp()
         domain = mail.maildir.MaildirDirdbmDomain(self.S, self.tmpdir)
         domain.addUser('user', 'password')
-        self.S.domains['test.domain'] = domain
+        self.S.addDomain('test.domain', domain)
 
     def tearDown(self):
         shutil.rmtree(self.tmpdir)
 
+
+    def testAddAliasableDomain(self):
+        """
+        Test that adding an IAliasableDomain to a mail service properly sets
+        up alias group references and such.
+        """
+        aliases = object()
+        domain = StubAliasableDomain()
+        self.S.aliases = aliases
+        self.S.addDomain('example.com', domain)
+        self.assertIdentical(domain.aliasGroup, aliases)
+
+
     def testReceivedHeader(self):
          hdr = self.D.receivedHeader(
              ('remotehost', '123.232.101.234'),
@@ -463,7 +537,7 @@ class VirtualPOP3TestCase(unittest.TestCase):
         self.S = mail.mail.MailService()
         self.D = mail.maildir.MaildirDirdbmDomain(self.S, self.tmpdir)
         self.D.addUser('user', 'password')
-        self.S.domains['test.domain'] = self.D
+        self.S.addDomain('test.domain', self.D)
 
         portal = cred.portal.Portal(self.D)
         map(portal.registerChecker, self.D.getCredentialsCheckers())
@@ -788,6 +862,31 @@ class MXTestCase(unittest.TestCase):
     def testSimpleFailureWithFallback(self):
         return self.assertFailure(self.mx.getMX('test.domain'), DNSLookupError)
 
+
+    def test_failureWithSuccessfulFallback(self):
+        """
+        Test that if the MX record lookup fails, fallback is enabled, and an A
+        record is available for the name, then the Deferred returned by
+        L{MXCalculator.getMX} ultimately fires with a Record_MX instance which
+        gives the address in the A record for the name.
+        """
+        class DummyResolver(object):
+            """
+            Fake resolver which will fail an MX lookup but then succeed a
+            getHostByName call.
+            """
+            def lookupMailExchange(self, domain):
+                return defer.fail(DNSNameError())
+
+            def getHostByName(self, domain):
+                return defer.succeed("1.2.3.4")
+
+        self.mx.resolver = DummyResolver()
+        d = self.mx.getMX("domain")
+        d.addCallback(self.assertEqual, Record_MX(name="1.2.3.4"))
+        return d
+
+
     def testManyRecords(self):
         self.auth.addresses['test.domain'] = [
             'mx1.test.domain', 'mx2.test.domain', 'mx3.test.domain'
@@ -826,7 +925,7 @@ class MXTestCase(unittest.TestCase):
         self.assertEqual(str(againMX.name), str(nextMX.name))
 
 class LiveFireExercise(unittest.TestCase):
-    if interfaces.IReactorUDP(reactor, default=None) is None:
+    if interfaces.IReactorUDP(reactor, None) is None:
         skip = "UDP support is required to determining MX records"
 
     def setUp(self):
@@ -847,8 +946,7 @@ class LiveFireExercise(unittest.TestCase):
         service.smtpPortal.registerChecker(cred.checkers.AllowAnonymousAccess())
         domain = mail.maildir.MaildirDirdbmDomain(service, 'domainDir')
         domain.addUser('user', 'password')
-        service.domains['test.domain'] = domain
-        service.portals['test.domain'] = cred.portal.Portal(domain)
+        service.addDomain('test.domain', domain)
         service.portals[''] = service.portals['test.domain']
         map(service.portals[''].registerChecker, domain.getCredentialsCheckers())
 
@@ -870,32 +968,28 @@ class LiveFireExercise(unittest.TestCase):
             'QUIT'
         ])
 
-        done = []
+        done = Deferred()
         f = protocol.ClientFactory()
         f.protocol = lambda: client
-        f.clientConnectionLost = lambda *args: done.append(None)
+        f.clientConnectionLost = lambda *args: done.callback(None)
         reactor.connectTCP('127.0.0.1', self.smtpServer.getHost().port, f)
 
-        i = 0
-        while len(done) == 0 and i < 1000:
-            reactor.iterate(0.01)
-            i += 1
+        def finished(ign):
+            mbox = domain.requestAvatar('user', None, pop3.IMailbox)[1]
+            msg = mbox.getMessage(0).read()
+            self.failIfEqual(msg.find('This is the message'), -1)
 
-        self.failUnless(done)
+            return self.smtpServer.stopListening()
+        done.addCallback(finished)
+        return done
 
-        mbox = domain.requestAvatar('user', None, pop3.IMailbox)[1]
-        msg = mbox.getMessage(0).read()
-        self.failIfEqual(msg.find('This is the message'), -1)
-
-        self.smtpServer.stopListening()
 
     def testRelayDelivery(self):
         # Here is the service we will connect to and send mail from
         insServ = mail.mail.MailService()
         insServ.smtpPortal.registerChecker(cred.checkers.AllowAnonymousAccess())
         domain = mail.maildir.MaildirDirdbmDomain(insServ, 'insertionDomain')
-        insServ.domains['insertion.domain'] = domain
-        insServ.portals['insertion.domain'] = cred.portal.Portal(domain)
+        insServ.addDomain('insertion.domain', domain)
         os.mkdir('insertionQueue')
         insServ.setQueue(mail.relaymanager.Queue('insertionQueue'))
         insServ.domains.setDefaultDomain(mail.relay.DomainQueuer(insServ))
@@ -916,8 +1010,7 @@ class LiveFireExercise(unittest.TestCase):
         destServ.smtpPortal.registerChecker(cred.checkers.AllowAnonymousAccess())
         domain = mail.maildir.MaildirDirdbmDomain(destServ, 'destinationDomain')
         domain.addUser('user', 'password')
-        destServ.domains['destination.domain'] = domain
-        destServ.portals['destination.domain'] = cred.portal.Portal(domain)
+        destServ.addDomain('destination.domain', domain)
         os.mkdir('destinationQueue')
         destServ.setQueue(mail.relaymanager.Queue('destinationQueue'))
         manager2 = mail.relaymanager.SmartHostSMTPRelayingManager(destServ.queue)
@@ -941,33 +1034,29 @@ class LiveFireExercise(unittest.TestCase):
             'QUIT'
         ])
 
-        done = []
+        done = Deferred()
         f = protocol.ClientFactory()
         f.protocol = lambda: client
-        f.clientConnectionLost = lambda *args: done.append(None)
+        f.clientConnectionLost = lambda *args: done.callback(None)
         reactor.connectTCP('127.0.0.1', self.insServer.getHost().port, f)
 
-        i = 0
-        while len(done) == 0 and i < 1000:
-            reactor.iterate(0.01)
-            i += 1
+        def finished(ign):
+            # First part of the delivery is done.  Poke the queue manually now
+            # so we don't have to wait for the queue to be flushed.
+            delivery = manager.checkState()
+            def delivered(ign):
+                mbox = domain.requestAvatar('user', None, pop3.IMailbox)[1]
+                msg = mbox.getMessage(0).read()
+                self.failIfEqual(msg.find('This is the message'), -1)
 
-        self.failUnless(done)
+                self.insServer.stopListening()
+                self.destServer.stopListening()
+                helper.stopService()
+            delivery.addCallback(delivered)
+            return delivery
+        done.addCallback(finished)
+        return done
 
-        # First part of the delivery is done.  Poke the queue manually now
-        # so we don't have to wait for the queue to be flushed.
-        manager.checkState()
-
-        for i in range(1000):
-            reactor.iterate(0.01)
-
-        mbox = domain.requestAvatar('user', None, pop3.IMailbox)[1]
-        msg = mbox.getMessage(0).read()
-        self.failIfEqual(msg.find('This is the message'), -1)
-
-        self.insServer.stopListening()
-        self.destServer.stopListening()
-        helper.stopService()
 
 aliasFile = StringIO.StringIO("""\
 # Here's a comment
@@ -1097,11 +1186,6 @@ class ProcessAliasTestCase(test_process.SignalMixin, unittest.TestCase):
     def tearDownClass(self):
         smtp.DNSNAME = self.DNSNAME
 
-    def tearDown(self):
-        reactor.iterate()
-        reactor.iterate()
-        reactor.iterate()
-
     def testProcessAlias(self):
         path = util.sibpath(__file__, 'process.alias.sh')
         a = mail.alias.ProcessAlias(path, None, None)
@@ -1181,7 +1265,7 @@ class ProcessAliasTestCase(test_process.SignalMixin, unittest.TestCase):
             mail.alias.MessageWrapper(DummyProcess(), 'echo')
         ])
 
-if interfaces.IReactorProcess(reactor, default=None) is None:
+if interfaces.IReactorProcess(reactor, None) is None:
     ProcessAliasTestCase = "IReactorProcess not supported"
 
 class TestDomain:
