@@ -16,14 +16,12 @@ import sys, operator
 from twisted.internet.protocol import ServerFactory, Protocol, ClientFactory
 from twisted.internet.interfaces import ITransport
 from twisted.internet import reactor, error
-from twisted.python import log, components
+from twisted.python import log
 from zope.interface import implements, providedBy, directlyProvides
 
 class ProtocolWrapper(Protocol):
     """Wraps protocol instances and acts as their transport as well."""
 
-    implements(ITransport)
-
     disconnecting = 0
 
     def __init__(self, factory, wrappedProtocol):
@@ -31,8 +29,7 @@ class ProtocolWrapper(Protocol):
         self.factory = factory
 
     def makeConnection(self, transport):
-        for iface in providedBy(transport):
-            directlyProvides(self, iface)
+        directlyProvides(self, *providedBy(self) + providedBy(transport))
         Protocol.makeConnection(self, transport)
 
     # Transport relaying
@@ -78,8 +75,6 @@ class ProtocolWrapper(Protocol):
         self.factory.unregisterProtocol(self)
         self.wrappedProtocol.connectionLost(reason)
 
-components.backwardsCompatImplements(ProtocolWrapper)
-
 
 class WrappingFactory(ClientFactory):
     """Wraps a factory and its protocols, and keeps track of them."""
@@ -146,10 +141,10 @@ class ThrottlingProtocol(ProtocolWrapper):
 
 
     def throttleReads(self):
-        self.transport.stopReading()
+        self.transport.pauseProducing()
 
     def unthrottleReads(self):
-        self.transport.startReading()
+        self.transport.resumeProducing()
 
     def throttleWrites(self):
         if hasattr(self, "producer"):
@@ -302,6 +297,8 @@ class LimitConnectionsByPeer(WrappingFactory):
 class LimitTotalConnectionsFactory(ServerFactory):
     """Factory that limits the number of simultaneous connections.
 
+    API Stability: Unstable
+
     @type connectionCount: C{int}
     @ivar connectionCount: number of current connections.
     @type connectionLimit: C{int} or C{None}
@@ -310,8 +307,6 @@ class LimitTotalConnectionsFactory(ServerFactory):
     @cvar overflowProtocol: Protocol to use for new connections when
         connectionLimit is exceeded.  If C{None} (the default value), excess
         connections will be closed immediately.
-
-    API Stability: Unstable
     """
     connectionCount = 0
     connectionLimit = None
@@ -506,6 +501,10 @@ class TimeoutMixin:
 
     __timeoutCall = None
 
+    def callLater(self, period, func):
+        return reactor.callLater(period, func)
+
+
     def resetTimeout(self):
         """Reset the timeout count down"""
         if self.__timeoutCall is not None and self.timeOut is not None:
@@ -528,7 +527,7 @@ class TimeoutMixin:
             else:
                 self.__timeoutCall.reset(period)
         elif period is not None:
-            self.__timeoutCall = reactor.callLater(period, self.__timedOut)
+            self.__timeoutCall = self.callLater(period, self.__timedOut)
 
         return prev
 
