@@ -266,7 +266,9 @@ def _getpass(prompt):
     except EOFError:
         raise KeyboardInterrupt
 
-def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
+def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0,
+                confirmPrompt = 'Confirm password: ',
+                mismatchMessage = "Passwords don't match."):
     """Obtain a password by prompting or from stdin.
 
     If stdin is a terminal, prompt for a new password, and confirm (if
@@ -276,7 +278,7 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
     If stdin is not a terminal, and C{forceTTY} is not true, read in a line
     and use it as the password, less the trailing newline, if any.  If
     C{forceTTY} is true, attempt to open a tty and prompt for the password
-    using it.  Raise a RuntimeException if this is not possible.
+    using it.  Raise a RuntimeError if this is not possible.
 
     @returns: C{str}
     """
@@ -290,7 +292,7 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
                     old = sys.stdin, sys.stdout
                     sys.stdin = sys.stdout = open('/dev/tty', 'r+')
                 except:
-                    raise RuntimeException, "Cannot obtain a TTY"
+                    raise RuntimeError("Cannot obtain a TTY")
             else:
                 password = sys.stdin.readline()
                 if password[-1] == '\n':
@@ -301,11 +303,11 @@ def getPassword(prompt = 'Password: ', confirm = 0, forceTTY = 0):
             try1 = _getpass(prompt)
             if not confirm:
                 return try1
-            try2 = _getpass("Confirm: ")
+            try2 = _getpass(confirmPrompt)
             if try1 == try2:
                 return try1
             else:
-                sys.stderr.write("Passwords don't match.\n")
+                sys.stderr.write(mismatchMessage + "\n")
     finally:
         if old:
             sys.stdin.close()
@@ -455,13 +457,13 @@ class IntervalDifferential:
     Given a list of intervals, generate the amount of time to sleep between
     \"instants\".
 
-    For example, given 7, 11 and 13, the three (infinite) sequences
+    For example, given 7, 11 and 13, the three (infinite) sequences::
 
         7 14 21 28 35 ...
         11 22 33 44 ...
         13 26 39 52 ...
 
-    will be generated, merged, and used to produce:
+    will be generated, merged, and used to produce::
 
         (7, 0) (4, 1) (2, 2) (1, 0) (7, 0) (1, 1) (4, 2) (2, 0) (5, 1) (2, 0)
 
@@ -472,7 +474,7 @@ class IntervalDifferential:
     def __init__(self, intervals, default=60):
         """
         @type intervals: C{list} of C{int}, C{long}, or C{float} param
-        @intervals: The intervals between instants.
+        @param intervals: The intervals between instants.
 
         @type default: C{int}, C{long}, or C{float}
         @param default: The duration to generate if the intervals list
@@ -784,11 +786,58 @@ def mergeFunctionMetadata(f, g):
             pass
     return g
 
+
+def nameToLabel(mname):
+    """
+    Convert a string like a variable name into a slightly more human-friendly
+    string with spaces and capitalized letters.
+
+    @type mname: C{str}
+    @param mname: The name to convert to a label.  This must be a string
+    which could be used as a Python identifier.  Strings which do not take
+    this form will result in unpredictable behavior.
+
+    @rtype: C{str}
+    """
+    labelList = []
+    word = ''
+    lastWasUpper = False
+    for letter in mname:
+        if letter.isupper() == lastWasUpper:
+            # Continuing a word.
+            word += letter
+        else:
+            # breaking a word OR beginning a word
+            if lastWasUpper:
+                # could be either
+                if len(word) == 1:
+                    # keep going
+                    word += letter
+                else:
+                    # acronym
+                    # we're processing the lowercase letter after the acronym-then-capital
+                    lastWord = word[:-1]
+                    firstLetter = word[-1]
+                    labelList.append(lastWord)
+                    word = firstLetter + letter
+            else:
+                # definitely breaking: lower to upper
+                labelList.append(word)
+                word = letter
+        lastWasUpper = letter.isupper()
+    if labelList:
+        labelList[0] = labelList[0].capitalize()
+    else:
+        return mname.capitalize()
+    labelList.append(word)
+    return ' '.join(labelList)
+
+
 __all__ = [
     "uniquify", "padTo", "getPluginDirs", "addPluginDir", "sibpath",
     "getPassword", "dict", "println", "keyed_md5", "makeStatBar",
     "OrderedDict", "InsensitiveDict", "spewer", "searchupwards", "LineLog",
     "raises", "IntervalDifferential", "FancyStrMixin", "FancyEqMixin",
     "dsu", "switchUID", "SubclassableCStringIO", "moduleMovedForSplit",
-    "unsignedID", "mergeFunctionMetadata",
+    "unsignedID", "mergeFunctionMetadata", "nameToLabel",
 ]
