@@ -27,15 +27,15 @@ __version__ = "$Revision: 1.32 $"[11:-2]
 # mechanisms (like XMLRPC)
 
 # system imports
-import types
-from zope.interface import implements
+import sys
+from zope.interface import implements, Interface
 
 # twisted imports
-from twisted.python import log, reflect, components
+from twisted.python import log, reflect
 
 # sibling imports
 from jelly import setUnjellyableForClass, setUnjellyableForClassTree, setUnjellyableFactoryForClass, unjellyableRegistry
-from jelly import Jellyable, Unjellyable, _Dummy
+from jelly import Jellyable, Unjellyable, _Dummy, _DummyNewStyle
 from jelly import setInstanceState, getInstanceState
 
 # compatibility
@@ -54,10 +54,10 @@ class NoSuchMethod(AttributeError):
     """Raised if there is no such remote method"""
 
 
-class IPBRoot(components.Interface):
+class IPBRoot(Interface):
     """Factory for root Referenceable objects for PB servers."""
 
-    def rootObject(self, broker):
+    def rootObject(broker):
         """Return root Referenceable for broker."""
 
 
@@ -153,8 +153,6 @@ class Root(Referenceable):
         """
         return self
 
-components.backwardsCompatImplements(Root)
-
 
 class ViewPoint(Referenceable):
     """
@@ -446,7 +444,10 @@ class RemoteCache(RemoteCopy, Serializable):
             return setInstanceState(self, unjellier, jellyList)
         self.broker = unjellier.invoker
         self.luid = jellyList[1]
-        cProxy = _Dummy()
+        if isinstance(self.__class__, type): #new-style class
+            cProxy = _DummyNewStyle()
+        else:
+            cProxy = _Dummy()
         cProxy.__class__ = self.__class__
         cProxy.__dict__ = self.__dict__
         # XXX questionable whether this was a good design idea...
@@ -479,7 +480,7 @@ class RemoteCache(RemoteCopy, Serializable):
     def __hash__(self):
         """Hash me.
         """
-        return id(self.__dict__)
+        return int(id(self.__dict__) % sys.maxint)
 
     broker = None
     luid = None
