@@ -1,22 +1,11 @@
 # -*- test-case-name: twisted.web2.test.test_http_headers -*-
 from __future__ import generators
 
-import types
+import types, time
 from calendar import timegm
-from time import gmtime, time
 import base64
 import re
 
-# Counterpart to evilness in test_http_headers
-try:
-    _http_headers_isBeingTested
-    from twisted.python.util import OrderedDict
-    ODict = OrderedDict
-    time = lambda : 999999990 # Sun, 09 Sep 2001 01:46:30 GMT
-except:
-    ODict = dict
-
-
 def dashCapitalize(s):
     ''' Capitalize a string, making sure to treat - as a word seperator '''
     return '-'.join([ x.capitalize() for x in s.split('-')])
@@ -29,6 +18,142 @@ monthname = [None,
              'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
 monthname_lower = [name and name.lower() for name in monthname]
 
+# HTTP Header parsing API
+
+header_case_mapping = {}
+
+def casemappingify(d):
+    global header_case_mapping
+    newd = dict([(key.lower(),key) for key in d.keys()])
+    header_case_mapping.update(newd)
+
+def lowerify(d):
+    return dict([(key.lower(),value) for key,value in d.items()])
+
+
+class HeaderHandler(object):
+    """HeaderHandler manages header generating and parsing functions.
+    """
+    HTTPParsers = {}
+    HTTPGenerators = {}
+
+    def __init__(self, parsers=None, generators=None):
+        """
+        @param parsers: A map of header names to parsing functions.
+        @type parsers: L{dict}
+
+        @param generators: A map of header names to generating functions.
+        @type generators: L{dict}
+        """
+
+        if parsers:
+            self.HTTPParsers.update(parsers)
+        if generators:
+            self.HTTPGenerators.update(generators)
+
+    def parse(self, name, header):
+        """
+        Parse the given header based on its given name.
+
+        @param name: The header name to parse.
+        @type name: C{str}
+        
+        @param header: A list of unparsed headers.
+        @type header: C{list} of C{str}
+
+        @return: The return value is the parsed header representation,
+            it is dependent on the header.  See the HTTP Headers document.
+        """
+        parser = self.HTTPParsers.get(name, None)
+        if parser is None:
+            raise ValueError("No header parser for header '%s', either add one or use getHeaderRaw." % (name,))
+
+        try:
+            for p in parser:
+                # print "Parsing %s: %s(%s)" % (name, repr(p), repr(h))
+                header = p(header)
+                # if isinstance(h, types.GeneratorType):
+                #     h=list(h)
+        except ValueError,v:
+            # print v
+            header=None
+
+        return header
+
+    def generate(self, name, header):
+        """
+        Generate the given header based on its given name.
+
+        @param name: The header name to generate.
+        @type name: C{str}
+        
+        @param header: A parsed header, such as the output of 
+            L{HeaderHandler}.parse.
+
+        @return: C{list} of C{str} each representing a generated HTTP header.
+        """
+        generator = self.HTTPGenerators.get(name, None)
+
+        if generator is None:
+            # print self.generators
+            raise ValueError("No header generator for header '%s', either add one or use setHeaderRaw." % (name,))
+
+        for g in generator:
+            header = g(header)
+
+        #self._raw_headers[name] = h
+        return header
+
+    def updateParsers(self, parsers):
+        """Update en masse the parser maps.
+
+        @param parsers: Map of header names to parser chains.
+        @type parsers: C{dict}
+        """
+        casemappingify(parsers)
+        self.HTTPParsers.update(lowerify(parsers))
+
+    def addParser(self, name, value):
+        """Add an individual parser chain for the given header.
+        
+        @param name: Name of the header to add
+        @type name: C{str}
+
+        @param value: The parser chain
+        @type value: C{str}
+        """
+        self.updateParsers({name: value})
+
+    def updateGenerators(self, generators):
+        """Update en masse the generator maps.
+
+        @param parsers: Map of header names to generator chains.
+        @type parsers: C{dict}
+        """
+        casemappingify(generators)
+        self.HTTPGenerators.update(lowerify(generators))
+
+    def addGenerators(self, name, value):
+        """Add an individual generator chain for the given header.
+
+        @param name: Name of the header to add
+        @type name: C{str}
+
+        @param value: The generator chain
+        @type value: C{str}
+        """
+        self.updateGenerators({name: value})
+
+    def update(self, parsers, generators):
+        """Conveniently update parsers and generators all at once.
+        """
+        self.updateParsers(parsers)
+        self.updateGenerators(generators)
+
+
+DefaultHTTPHandler = HeaderHandler()
+
+
 ## HTTP DateTime parser
 def parseDateTime(dateString):
     """Convert an HTTP date string (one of three formats) to seconds since epoch."""
@@ -73,13 +198,14 @@ def parseDateTime(dateString):
         time = parts[3]
     else:
         raise ValueError("Unknown datetime format %r" % dateString)
-    
+
     day = int(day)
     month = int(monthname_lower.index(month.lower()))
     year = int(year)
     hour, min, sec = map(int, time.split(':'))
     return int(timegm((year, month, day, hour, min, sec)))
 
+
 ##### HTTP tokenizer
 class Token(str):
     __slots__=[]
@@ -101,21 +227,21 @@ def tokenize(header, foldCase=True):
     """Tokenize a string according to normal HTTP header parsing rules.
 
     In particular:
-    - Whitespace is irrelevant and eaten next to special separator tokens.
-      Its existance (but not amount) is important between character strings.
-    - Quoted string support including embedded backslashes.
-    - Case is insignificant (and thus lowercased), except in quoted strings.
-       (unless foldCase=False)
-    - Multiple headers are concatenated with ','
-    
+     - Whitespace is irrelevant and eaten next to special separator tokens.
+       Its existance (but not amount) is important between character strings.
+     - Quoted string support including embedded backslashes.
+     - Case is insignificant (and thus lowercased), except in quoted strings.
+        (unless foldCase=False)
+     - Multiple headers are concatenated with ','
+
     NOTE: not all headers can be parsed with this function.
-    
+
     Takes a raw header value (list of strings), and
     Returns a generator of strings and Token class instances.
     """
     tokens=http_tokens
     ctls=http_ctls
-    
+
     string = ",".join(header)
     list = []
     start = 0
@@ -124,7 +250,7 @@ def tokenize(header, foldCase=True):
     qpair = False
     inSpaces = -1
     qstring = None
-    
+
     for x in string:
         if quoted:
             if qpair:
@@ -144,7 +270,7 @@ def tokenize(header, foldCase=True):
                     yield string[start:cur].lower()
                 else:
                     yield string[start:cur]
-                
+
             start = cur+1
             if x == '"':
                 quoted = True
@@ -162,15 +288,15 @@ def tokenize(header, foldCase=True):
             if inSpaces is True:
                 yield Token(' ')
                 inSpaces = False
-                
+
             inSpaces = False
         cur = cur+1
-        
+
     if qpair:
         raise ValueError, "Missing character after '\\'"
     if quoted:
         raise ValueError, "Missing end quote"
-    
+
     if start != cur:
         if foldCase:
             yield string[start:cur].lower()
@@ -180,28 +306,28 @@ def tokenize(header, foldCase=True):
 def split(seq, delim):
     """The same as str.split but works on arbitrary sequences.
     Too bad it's not builtin to python!"""
-    
+
     cur = []
     for item in seq:
         if item == delim:
             yield cur
-            cur = []
+            cur = []        
         else:
             cur.append(item)
     yield cur
 
 # def find(seq, *args):
-#     """The same as seq.index but returns -1 if not found, instead 
+#     """The same as seq.index but returns -1 if not found, instead
 #     Too bad it's not builtin to python!"""
 #     try:
 #         return seq.index(value, *args)
 #     except ValueError:
 #         return -1
-    
+
 
 def filterTokens(seq):
     """Filter out instances of Token, leaving only a list of strings.
-    
+
     Used instead of a more specific parsing method (e.g. splitting on commas)
     when only strings are expected, so as to be a little lenient.
 
@@ -221,7 +347,7 @@ def checkSingleToken(tokens):
     if len(tokens) != 1:
         raise ValueError, "Expected single token, not %s." % (tokens,)
     return tokens[0]
-    
+
 def parseKeyValue(val):
     if len(val) == 1:
         return val[0],None
@@ -248,7 +374,7 @@ def listParser(fun):
 
 def last(seq):
     """Return seq[-1]"""
-    
+
     return seq[-1]
 
 ##### Generation utilities
@@ -260,7 +386,7 @@ def listGenerator(fun):
     the given list, then joins the result with generateList"""
     def listGeneratorHelper(l):
         return generateList([fun(e) for e in l])
-            
+
     return listGeneratorHelper
 
 def generateList(seq):
@@ -280,8 +406,26 @@ def generateKeyValues(kvs):
     return ";".join(l)
 
 
-class MimeType:
+class MimeType(object):
+    def fromString(klass, mimeTypeString):
+        """Generate a MimeType object from the given string.
+
+        @param mimeTypeString: The mimetype to parse
+
+        @return: L{MimeType}
+        """
+        return DefaultHTTPHandler.parse('content-type', [mimeTypeString])
+
+    fromString = classmethod(fromString)
+
     def __init__(self, mediaType, mediaSubtype, params={}, **kwargs):
+        """
+        @type mediaType: C{str}
+
+        @type mediaSubtype: C{str}
+
+        @type params: C{dict}
+        """
         self.mediaType = mediaType
         self.mediaSubtype = mediaSubtype
         self.params = dict(params)
@@ -290,8 +434,8 @@ class MimeType:
             self.params.update(kwargs)
 
     def __eq__(self, other):
-        return (isinstance(other, MimeType) and
-                self.mediaType == other.mediaType and
+        if not isinstance(other, MimeType): return NotImplemented
+        return (self.mediaType == other.mediaType and
                 self.mediaSubtype == other.mediaSubtype and
                 self.params == other.params)
 
@@ -303,18 +447,18 @@ class MimeType:
 
     def __hash__(self):
         return hash(self.mediaType)^hash(self.mediaSubtype)^hash(tuple(self.params.iteritems()))
-    
+
 ##### Specific header parsers.
 def parseAccept(field):
     type,args = parseArgs(field)
 
     if len(type) != 3 or type[1] != Token('/'):
         raise ValueError, "MIME Type "+str(type)+" invalid."
-    
+
     # okay, this spec is screwy. A 'q' parameter is used as the separator
     # between MIME parameters and (as yet undefined) additional HTTP
     # parameters.
-    
+
     num = 0
     for arg in args:
         if arg[0] == 'q':
@@ -328,7 +472,7 @@ def parseAccept(field):
 
     # Default values for parameters:
     qval = 1.0
-    
+
     # Parse accept parameters:
     for param in params:
         if param[0] =='q':
@@ -342,9 +486,9 @@ def parseAccept(field):
 
 def parseAcceptQvalue(field):
     type,args=parseArgs(field)
-    
+
     type = checkSingleToken(type)
-    
+
     qvalue = 1.0 # Default qvalue is 1
     for arg in args:
         if arg[0] == 'q':
@@ -368,14 +512,14 @@ def parseContentType(header):
     # Case folding is disabled for this header, because of use of
     # Content-Type: multipart/form-data; boundary=CaSeFuLsTuFf
     # So, we need to explicitly .lower() the type/subtype and arg keys.
-    
+
     type,args = parseArgs(header)
-    
+
     if len(type) != 3 or type[1] != Token('/'):
         raise ValueError, "MIME Type "+str(type)+" invalid."
-    
+
     args = [(kv[0].lower(), kv[1]) for kv in args]
-        
+
     return MimeType(type[0].lower(), type[2].lower(), tuple(args))
 
 def parseContentMD5(header):
@@ -383,10 +527,10 @@ def parseContentMD5(header):
         return base64.decodestring(header)
     except Exception,e:
         raise ValueError(e)
-    
+
 def parseContentRange(header):
     """Parse a content-range header into (kind, start, end, realLength).
-    
+
     realLength might be None if real length is not known ('*').
     start and end might be None if start,end unspecified (for response code 416)
     """
@@ -406,14 +550,14 @@ def parseContentRange(header):
 
 def parseExpect(field):
     type,args=parseArgs(field)
-    
+
     type=parseKeyValue(type)
     return (type[0], (lambda *args:args)(type[1], *args))
 
 def parseExpires(header):
     # """HTTP/1.1 clients and caches MUST treat other invalid date formats,
     #    especially including the value 0, as in the past (i.e., "already expired")."""
-    
+
     try:
         return parseDateTime(header)
     except ValueError:
@@ -424,7 +568,7 @@ def parseIfModifiedSince(header):
     #   If-Modified-Since: Thu, 05 Aug 2004 12:57:27 GMT; length=123
     # which is blantantly RFC-violating and not documented anywhere
     # except bug-trackers for web frameworks.
-    
+
     # So, we'll just strip off everything after a ';'.
     return parseDateTime(header.split(';', 1)[0])
 
@@ -433,31 +577,31 @@ def parseIfRange(headers):
         return ETag.parse(tokenize(headers))
     except ValueError:
         return parseDateTime(last(headers))
-    
+
 def parseRange(range):
     range = list(range)
     if len(range) < 3 or range[1] != Token('='):
         raise ValueError("Invalid range header format: %s" %(range,))
-    
+
     type=range[0]
     if type != 'bytes':
         raise ValueError("Unknown range unit: %s." % (type,))
     rangeset=split(range[2:], Token(','))
     ranges = []
-    
+
     for byterangespec in rangeset:
         if len(byterangespec) != 1:
             raise ValueError("Invalid range header format: %s" % (range,))
         start,end=byterangespec[0].split('-')
-        
+
         if not start and not end:
             raise ValueError("Invalid range header format: %s" % (range,))
-        
+
         if start:
             start = int(start)
         else:
             start = None
-        
+
         if end:
             end = int(end)
         else:
@@ -471,43 +615,61 @@ def parseRange(range):
 def parseRetryAfter(header):
     try:
         # delta seconds
-        return time() + int(header)
+        return time.time() + int(header)
     except ValueError:
         # or datetime
         return parseDateTime(header)
 
 # WWW-Authenticate and Authorization
 
-def parseWWWAuthenticate(header):
-    scheme, rest = split(header, Token(' '))
-    challenge = [parseKeyValue(arg) for arg in split(rest, Token(','))]
-    
-    return [scheme[0], ODict(challenge)]
+def parseWWWAuthenticate(tokenized):
+    headers = []
 
-def parseBasicAuthorization(header):
-    scheme, rest = header[0].split(' ', 1)
-    # this header isn't tokenized because it may eat characters
-    # in the unquoted base64 encoded credentials
-    # but to be sure it gets the same style output we manually
-    # lowercase the scheme
-    return scheme.lower(), rest
+    tokenList = list(tokenized)
 
-# we need to farm out authorization schemes to their
-# own parser chains
+    while tokenList:
+        scheme = tokenList.pop(0)
+        challenge = {}
+        last = None
+        kvChallenge = False
 
-authParsers = {'digest': (tokenize, parseWWWAuthenticate),
-               'basic': (parseBasicAuthorization,)}                         
+        while tokenList:
+            token = tokenList.pop(0)
+            if token == Token('='):
+                kvChallenge = True
+                challenge[last] = tokenList.pop(0)
+                last = None
+                
+            elif token == Token(','):
+                if kvChallenge:
+                    if len(tokenList) > 1 and tokenList[1] != Token('='):
+                        break
+
+                else:
+                    break
+
+            else:
+                last = token
+        
+        if last and scheme and not challenge and not kvChallenge:
+            challenge = last
+            last = None
+
+        headers.append((scheme, challenge))
+        
+    if last and last not in (Token('='), Token(',')):
+        if headers[-1] == (scheme, challenge):
+            scheme = last
+            challenge = {}
+            headers.append((scheme, challenge))
+
+    return headers
 
 def parseAuthorization(header):
-    header = tuple(header)
-    h = zip(*split(tokenize(header), Token(' ')))[0]
-    scheme, rest = h[0], h[1:]
-    
-    parsers = authParsers.get(scheme, [])
-    for p in parsers:
-        header = p(header)
-
-    return header
+    scheme, rest = header.split(' ', 1)
+    # this header isn't tokenized because it may eat characters
+    # in the unquoted base64 encoded credentials
+    return scheme.lower(), rest
 
 #### Header generators
 def generateAccept(accept):
@@ -516,10 +678,10 @@ def generateAccept(accept):
     out="%s/%s"%(mimeType.mediaType, mimeType.mediaSubtype)
     if mimeType.params:
         out+=';'+generateKeyValues(mimeType.params.iteritems())
-    
+
     if q != 1.0:
         out+=(';q=%.3f' % (q,)).rstrip('0').rstrip('.')
-        
+
     return out
 
 def removeDefaultEncoding(seq):
@@ -532,7 +694,7 @@ def generateAcceptQvalue(keyvalue):
         return "%s" % keyvalue[0:1]
     else:
         return ("%s;q=%.3f" % keyvalue).rstrip('0').rstrip('.')
-    
+
 def parseCacheControl(kv):
     k, v = parseKeyValue(kv)
     if k == 'max-age' or k == 'min-fresh' or k == 's-maxage':
@@ -574,12 +736,12 @@ def generateContentRange(tup):
         startend = '*'
     else:
         startend = '%d-%d' % (start, end)
-    
+
     return '%s %s/%s' % (type, startend, len)
 
 def generateDateTime(secSinceEpoch):
     """Convert seconds since epoch to HTTP datetime string."""
-    year, month, day, hh, mm, ss, wd, y, z = gmtime(secSinceEpoch)
+    year, month, day, hh, mm, ss, wd, y, z = time.gmtime(secSinceEpoch)
     s = "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (
         weekdayname[wd],
         day, monthname[month], year,
@@ -600,19 +762,19 @@ def generateRange(range):
         if s is None:
             return ''
         return s
-    
+
     type,ranges=range
-    
+
     if type != 'bytes':
         raise ValueError("Unknown range unit: "+type+".")
 
     return (type+'='+
             ','.join(['%s-%s' % (noneOr(startend[0]), noneOr(startend[1]))
                       for startend in ranges]))
-    
+
 def generateRetryAfter(when):
     # always generate delta seconds format
-    return str(int(when - time()))
+    return str(int(when - time.time()))
 
 def generateContentType(mimeType):
     out="%s/%s"%(mimeType.mediaType, mimeType.mediaSubtype)
@@ -628,25 +790,31 @@ def generateIfRange(dateOrETag):
 
 # WWW-Authenticate and Authorization
 
-def generateWWWAuthenticate(seq):
-    scheme, challenge = seq[0], ODict(seq[1])
-    l = []
-    for k,v in challenge.iteritems():
-        l.append("%s=%s" % (k, quoteString(v)))
+def generateWWWAuthenticate(headers):
+    _generated = []
+    for seq in headers:
+        scheme, challenge = seq[0], seq[1]
+        
+        # If we're going to parse out to something other than a dict
+        # we need to be able to generate from something other than a dict
 
-    return ["%s %s" % (scheme, ", ".join(l))]
+        try:
+            l = []
+            for k,v in dict(challenge).iteritems():
+                l.append("%s=%s" % (k, quoteString(v)))
 
-authGenerators = {'digest': (generateWWWAuthenticate,),
-                  'basic': (lambda s: [' '.join(s)],)}
+            _generated.append("%s %s" % (scheme, ", ".join(l)))
+        except ValueError:
+            _generated.append("%s %s" % (scheme, challenge))
+
+    return _generated
 
 def generateAuthorization(seq):
-    for p in authGenerators[seq[0].lower()]:
-        seq = p(seq)
+    return [' '.join(seq)]
+
 
-    return seq
-    
 ####
-class ETag:
+class ETag(object):
     def __init__(self, tag, weak=False):
         self.tag = str(tag)
         self.weak = weak
@@ -655,38 +823,38 @@ class ETag:
         # Sec 13.3.
         # The strong comparison function: in order to be considered equal, both
         #   validators MUST be identical in every way, and both MUST NOT be weak.
-        # 
+        #
         # The weak comparison function: in order to be considered equal, both
         #   validators MUST be identical in every way, but either or both of
-        #   them MAY be tagged as "weak" without affecting the result. 
-        
+        #   them MAY be tagged as "weak" without affecting the result.
+
         if not isinstance(other, ETag) or other.tag != self.tag:
             return False
-        
+
         if strongCompare and (other.weak or self.weak):
             return False
         return True
-    
+
     def __eq__(self, other):
         return isinstance(other, ETag) and other.tag == self.tag and other.weak == self.weak
-    
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
     def __repr__(self):
         return "Etag(%r, weak=%r)" % (self.tag, self.weak)
-    
+
     def parse(tokens):
         tokens=tuple(tokens)
         if len(tokens) == 1 and not isinstance(tokens[0], Token):
             return ETag(tokens[0])
-        
+
         if(len(tokens) == 3 and tokens[0] == "w"
            and tokens[1] == Token('/')):
             return ETag(tokens[2], weak=True)
-        
+
         raise ValueError("Invalid ETag.")
-            
+
     parse=staticmethod(parse)
 
     def generate(self):
@@ -748,7 +916,7 @@ class Cookie(object):
                 other.comment == self.comment and
                 other.commenturl == self.commenturl and
                 other.version == self.version)
-    
+
     def __ne__(self, other):
         return not self.__eq__(other)
 
@@ -760,12 +928,12 @@ def parseCookie(headers):
     Version 0) http://wp.netscape.com/newsref/std/cookie_spec.html
     Version 1) http://www.faqs.org/rfcs/rfc2965.html
     """
-    
+
     cookies = []
     # There can't really be multiple cookie headers according to RFC, because
     # if multiple headers are allowed, they must be joinable with ",".
     # Neither new RFC2965 cookies nor old netscape cookies are.
-    
+
     header = ';'.join(headers)
     if header[0:8].lower() == "$version":
         # RFC2965 cookie
@@ -781,7 +949,7 @@ def parseCookie(headers):
                 else:
                     (name,), = nameval
                     value = None
-                
+
                 name=name.lower()
                 if name == '$version':
                     continue
@@ -809,7 +977,7 @@ def parseCookie(headers):
             name,value = r_cookie.split('=', 1)
             name=name.strip(' \t')
             value=value.strip(' \t')
-            
+
             cookies.append(Cookie(name, value))
 
     return cookies
@@ -827,27 +995,27 @@ def generateCookie(cookies):
     # V1.
 
     # I choose to send them all as V1.
-    
+
     # You might think converting a V0 cookie to a V1 cookie would be lossless,
     # but you'd be wrong. If you do the conversion, and a V0 parser tries to
     # read the cookie, it will see a modified form of the cookie, in cases
-    # where quotes must be added to conform to proper V1 syntax. 
+    # where quotes must be added to conform to proper V1 syntax.
     # (as a real example: "Cookie: cartcontents=oid:94680,qty:1,auto:0,esp:y")
-    
+
     # However, that is what we will do, anyways. It has a high probability of
     # breaking applications that only handle oldstyle cookies, where some other
     # application set a newstyle cookie that is applicable over for site
     # (or host), AND where the oldstyle cookie uses a value which is invalid
-    # syntax in a newstyle cookie. 
-    
+    # syntax in a newstyle cookie.
+
     # Also, the cookie name *cannot* be quoted in V1, so some cookies just
     # cannot be converted at all. (e.g. "Cookie: phpAds_capAd[32]=2"). These
     # are just dicarded during conversion.
-    
+
     # As this is an unsolvable problem, I will pretend I can just say
     # OH WELL, don't do that, or else upgrade your old applications to have
     # newstyle cookie parsers.
-    
+
     # I will note offhandedly that there are *many* sites which send V0 cookies
     # that are not valid V1 cookie syntax. About 20% for my cookies file.
     # However, they do not generally mix them with V1 cookies, so this isn't
@@ -872,14 +1040,14 @@ def generateCookie(cookies):
                 # If it isn't a valid name, ignore the cookie.
                 # If it isn't a valid value, quote it and hope for the best on
                 # the other side.
-                
+
                 if cookie_validname_re.match(cookie.name) is None:
                     continue
 
                 value=cookie.value
                 if cookie_validvalue_re.match(cookie.value) is None:
                     value = quoteString(value)
-                    
+
                 str_cookies.append("%s=%s" % (cookie.name, value))
             else:
                 # V1 cookie, nice and easy
@@ -942,10 +1110,10 @@ def makeCookieFromList(tup, netscapeFormat):
         raise ValueError("Invalid cookie name: %r, starts with '$'." % name)
     cookie = Cookie(name, value)
     hadMaxAge = False
-    
+
     for name,value in tup[1:]:
         name = name.lower()
-        
+
         if value is None:
             if name in ("discard", "secure"):
                 # Boolean attrs
@@ -953,7 +1121,7 @@ def makeCookieFromList(tup, netscapeFormat):
             elif name != "port":
                 # Can be either boolean or explicit
                 continue
-        
+
         if name in ("comment", "commenturl", "discard", "domain", "path", "secure"):
             # simple cases
             setattr(cookie, name, value)
@@ -963,7 +1131,7 @@ def makeCookieFromList(tup, netscapeFormat):
             cookie.expires = parseDateTime(value)
         elif name == "max-age":
             hadMaxAge = True
-            cookie.expires = int(value) + time()
+            cookie.expires = int(value) + time.time()
         elif name == "port":
             if value is None:
                 cookie.ports = ()
@@ -973,9 +1141,9 @@ def makeCookieFromList(tup, netscapeFormat):
                 cookie.ports = tuple([int(s) for s in value.split(',')])
         elif name == "version":
             cookie.version = int(value)
-    
+
     return cookie
-        
+
 
 def generateSetCookie(cookies):
     setCookies = []
@@ -1006,7 +1174,7 @@ def generateSetCookie2(cookies):
         if cookie.domain:
             out.append("Domain=%s" % quoteString(cookie.domain))
         if cookie.expires:
-            out.append("Max-Age=%s" % (cookie.expires - time()))
+            out.append("Max-Age=%s" % (cookie.expires - time.time()))
         if cookie.path:
             out.append("Path=%s" % quoteString(cookie.path))
         if cookie.ports is not None:
@@ -1020,6 +1188,24 @@ def generateSetCookie2(cookies):
         setCookies.append('; '.join(out))
     return setCookies
 
+def parseDepth(depth):
+    if depth not in ("0", "1", "infinity"):
+        raise ValueError("Invalid depth header value: %s" % (depth,))
+    return depth
+
+def parseOverWrite(overwrite):
+    if overwrite == "F":
+        return False
+    elif overwrite == "T":
+        return True
+    raise ValueError("Invalid overwrite header value: %s" % (overwrite,))
+
+def generateOverWrite(overwrite):
+    if overwrite:
+        return "T"
+    else:
+        return "F"
+
 ##### Random stuff that looks useful.
 # def sortMimeQuality(s):
 #     def sorter(item1, item2):
@@ -1036,8 +1222,8 @@ def generateSetCookie2(cookies):
 #             return 1
 #         if item1[0] == item2[0]:
 #             return 0
-            
-            
+
+
 # def getMimeQuality(mimeType, accepts):
 #     type,args = parseArgs(mimeType)
 #     type=type.split(Token('/'))
@@ -1048,7 +1234,7 @@ def generateSetCookie2(cookies):
 #         accept,acceptQual=accept
 #         acceptType=accept[0:1]
 #         acceptArgs=accept[2]
-        
+
 #         if ((acceptType == type or acceptType == (type[0],'*') or acceptType==('*','*')) and
 #             (args == acceptArgs or len(acceptArgs) == 0)):
 #             return acceptQual
@@ -1057,7 +1243,7 @@ def generateSetCookie2(cookies):
 #     qual = accepts.get(type)
 #     if qual is not None:
 #         return qual
-    
+
 #     return accepts.get('*')
 
 # Headers object
@@ -1067,18 +1253,14 @@ class __RecalcNeeded(object):
 
 _RecalcNeeded = __RecalcNeeded()
 
-DefaultHTTPParsers = {}
-DefaultHTTPGenerators = {}
-
-class Headers:
+class Headers(object):
     """This class stores the HTTP headers as both a parsed representation and
     the raw string representation. It converts between the two on demand."""
-    
-    def __init__(self, headers=None, rawHeaders=None, parsers=DefaultHTTPParsers, generators=DefaultHTTPGenerators):
-        self._raw_headers = ODict()
-        self._headers = ODict()
-        self.parsers = parsers
-        self.generators = generators
+
+    def __init__(self, headers=None, rawHeaders=None, handler=DefaultHTTPHandler):
+        self._raw_headers = {}
+        self._headers = {}
+        self.handler = handler
         if headers is not None:
             for key, value in headers.iteritems():
                 self.setHeader(key, value)
@@ -1089,45 +1271,26 @@ class Headers:
     def _setRawHeaders(self, headers):
         self._raw_headers = headers
         self._headers = {}
-        
-    def _toParsed(self, name):
-        parser = self.parsers.get(name, None)
-        if parser is None:
-            raise ValueError("No header parser for header '%s', either add one or use getHeaderRaw." % (name,))
 
-        h = self._raw_headers[name]
-        try:
-            for p in parser:
-                # print "Parsing %s: %s(%s)" % (name, repr(p), repr(h))
-                h = p(h)
-                # if isinstance(h, types.GeneratorType):
-                #     h=list(h)
-        except ValueError,v:
-            # print v
-            h=None
-        
-        self._headers[name] = h
+    def _toParsed(self, name):
+        r = self._raw_headers.get(name, None)
+        h = self.handler.parse(name, r)
+        if h is not None:
+            self._headers[name] = h
         return h
 
     def _toRaw(self, name):
-        generator = self.generators.get(name, None)
+        h = self._headers.get(name, None)
+        r = self.handler.generate(name, h)
+        if r is not None:
+            self._raw_headers[name] = r
+        return r
 
-        if generator is None:
-            # print self.generators
-            raise ValueError("No header generator for header '%s', either add one or use setHeaderRaw." % (name,))
-
-        h = self._headers[name]
-        for g in generator:
-            h = g(h)
-            
-        self._raw_headers[name] = h
-        return h
-    
     def hasHeader(self, name):
         """Does a header with the given name exist?"""
         name=name.lower()
         return self._raw_headers.has_key(name)
-    
+
     def getRawHeaders(self, name, default=None):
         """Returns a list of headers matching the given name as the raw string given."""
 
@@ -1137,13 +1300,13 @@ class Headers:
             return raw_header
 
         return self._toRaw(name)
-    
+
     def getHeader(self, name, default=None):
-        """Returns the parsed representation of the given header.
+        """Ret9urns the parsed representation of the given header.
         The exact form of the return value depends on the header in question.
-        
+
         If no parser for the header exists, raise ValueError.
-        
+
         If the header doesn't exist, return default (or None if not specified)
         """
         name=name.lower()
@@ -1151,7 +1314,7 @@ class Headers:
         if parsed is not _RecalcNeeded:
             return parsed
         return self._toParsed(name)
-    
+
     def setRawHeaders(self, name, value):
         """Sets the raw representation of the given header.
         Value should be a list of strings, each being one header of the
@@ -1169,7 +1332,7 @@ class Headers:
         name=name.lower()
         self._raw_headers[name] = _RecalcNeeded
         self._headers[name] = value
-    
+
     def addRawHeader(self, name, value):
         """
         Add a raw value to a header that may or may not already exist.
@@ -1184,13 +1347,13 @@ class Headers:
             self._raw_headers[name] = raw_header
         elif raw_header is _RecalcNeeded:
             raw_header = self._toRaw(name)
-        
+
         raw_header.append(value)
         self._headers[name] = _RecalcNeeded
 
     def removeHeader(self, name):
         """Removes the header named."""
-        
+
         name=name.lower()
         if self._raw_headers.has_key(name):
             del self._raw_headers[name]
@@ -1203,7 +1366,7 @@ class Headers:
         """Return the name with the canonical capitalization, if known,
         otherwise, Caps-After-Dashes"""
         return header_case_mapping.get(name) or dashCapitalize(name)
-    
+
     def getAllRawHeaders(self):
         """Return an iterator of key,value pairs of all headers
         contained in this object, as strings. The keys are capitalized
@@ -1234,7 +1397,7 @@ iteritems = lambda x: x.iteritems()
 
 
 parser_general_headers = {
-    'Cache-Control':(tokenize, listParser(parseCacheControl), ODict),
+    'Cache-Control':(tokenize, listParser(parseCacheControl), dict),
     'Connection':(tokenize,filterTokens),
     'Date':(last,parseDateTime),
 #    'Pragma':tokenize
@@ -1258,13 +1421,13 @@ generator_general_headers = {
 }
 
 parser_request_headers = {
-    'Accept': (tokenize, listParser(parseAccept), ODict),
-    'Accept-Charset': (tokenize, listParser(parseAcceptQvalue), ODict, addDefaultCharset),
-    'Accept-Encoding':(tokenize, listParser(parseAcceptQvalue), ODict, addDefaultEncoding),
-    'Accept-Language':(tokenize, listParser(parseAcceptQvalue), ODict),
-    'Authorization': (parseAuthorization,),
+    'Accept': (tokenize, listParser(parseAccept), dict),
+    'Accept-Charset': (tokenize, listParser(parseAcceptQvalue), dict, addDefaultCharset),
+    'Accept-Encoding':(tokenize, listParser(parseAcceptQvalue), dict, addDefaultEncoding),
+    'Accept-Language':(tokenize, listParser(parseAcceptQvalue), dict),
+    'Authorization': (last, parseAuthorization),
     'Cookie':(parseCookie,),
-    'Expect':(tokenize, listParser(parseExpect), ODict),
+    'Expect':(tokenize, listParser(parseExpect), dict),
     'From':(last,),
     'Host':(last,),
     'If-Match':(tokenize, listParser(parseStarOrETag), list),
@@ -1276,11 +1439,10 @@ parser_request_headers = {
 #    'Proxy-Authorization':str, # what is "credentials"
     'Range':(tokenize, parseRange),
     'Referer':(last,str), # TODO: URI object?
-    'TE':(tokenize, listParser(parseAcceptQvalue), ODict),
+    'TE':(tokenize, listParser(parseAcceptQvalue), dict),
     'User-Agent':(last,str),
 }
 
-
 generator_request_headers = {
     'Accept': (iteritems,listGenerator(generateAccept),singleHeader),
     'Accept-Charset': (iteritems, listGenerator(generateAcceptQvalue),singleHeader),
@@ -1308,21 +1470,22 @@ parser_response_headers = {
     'Accept-Ranges':(tokenize, filterTokens),
     'Age':(last,int),
     'ETag':(tokenize, ETag.parse),
-    'Location':(last,), # TODO: uri object?
+    'Location':(last,), # TODO: URI object?
 #    'Proxy-Authenticate'
     'Retry-After':(last, parseRetryAfter),
     'Server':(last,),
     'Set-Cookie':(parseSetCookie,),
     'Set-Cookie2':(tokenize, parseSetCookie2),
     'Vary':(tokenize, filterTokens),
-    'WWW-Authenticate': (tokenize, parseWWWAuthenticate)
+    'WWW-Authenticate': (lambda h: tokenize(h, foldCase=False), 
+                         parseWWWAuthenticate,)
 }
 
 generator_response_headers = {
     'Accept-Ranges':(generateList, singleHeader),
     'Age':(str, singleHeader),
     'ETag':(ETag.generate, singleHeader),
-    'Location':(str, singleHeader), 
+    'Location':(str, singleHeader),
 #    'Proxy-Authenticate'
     'Retry-After':(generateRetryAfter, singleHeader),
     'Server':(str, singleHeader),
@@ -1358,31 +1521,43 @@ generator_entity_headers = {
     'Last-Modified':(generateDateTime, singleHeader),
     }
 
-DefaultHTTPParsers.update(parser_general_headers)
-DefaultHTTPParsers.update(parser_request_headers)
-DefaultHTTPParsers.update(parser_response_headers)
-DefaultHTTPParsers.update(parser_entity_headers)
+parser_dav_headers = {
+    'DAV'         : (tokenize, list),
+    'Depth'       : (last, parseDepth),
+    'Destination' : (last,), # TODO: URI object?
+   #'If'          : (),
+   #'Lock-Token'  : (),
+    'Overwrite'   : (last, parseOverWrite),
+   #'Status-URI'  : (),
+   #'Timeout'     : (),
+}
 
-DefaultHTTPGenerators.update(generator_general_headers)
-DefaultHTTPGenerators.update(generator_request_headers)
-DefaultHTTPGenerators.update(generator_response_headers)
-DefaultHTTPGenerators.update(generator_entity_headers)
+generator_dav_headers = {
+    'DAV'         : (generateList, singleHeader),
+    'Depth'       : (singleHeader),
+    'Destination' : (singleHeader),
+   #'If'          : (),
+   #'Lock-Token'  : (),
+    'Overwrite'   : (),
+   #'Status-URI'  : (),
+   #'Timeout'     : (),
+}
 
-header_case_mapping = {}
+DefaultHTTPHandler.updateParsers(parser_general_headers)
+DefaultHTTPHandler.updateParsers(parser_request_headers)
+DefaultHTTPHandler.updateParsers(parser_response_headers)
+DefaultHTTPHandler.updateParsers(parser_entity_headers)
+DefaultHTTPHandler.updateParsers(parser_dav_headers)
 
-def casemappingify(d):
-    global header_case_mapping
-    newd = dict([(key.lower(),key) for key in d.keys()])
-    header_case_mapping.update(newd)
-
-def lowerify(d):
-    newd = dict([(key.lower(),value) for key,value in d.items()])
-    d.clear()
-    d.update(newd)
+DefaultHTTPHandler.updateGenerators(generator_general_headers)
+DefaultHTTPHandler.updateGenerators(generator_request_headers)
+DefaultHTTPHandler.updateGenerators(generator_response_headers)
+DefaultHTTPHandler.updateGenerators(generator_entity_headers)
+DefaultHTTPHandler.updateGenerators(generator_dav_headers)
 
 
-casemappingify(DefaultHTTPParsers)
-casemappingify(DefaultHTTPGenerators)
+# casemappingify(DefaultHTTPParsers)
+# casemappingify(DefaultHTTPGenerators)
 
-lowerify(DefaultHTTPParsers)
-lowerify(DefaultHTTPGenerators)
+# lowerify(DefaultHTTPParsers)
+# lowerify(DefaultHTTPGenerators)
