@@ -13,6 +13,12 @@
 ##############################################################################
 """Zope application package.
 
-$Id: __init__.py 67630 2006-04-27 00:54:03Z jim $
+$Id: __init__.py 71649 2006-12-23 14:17:45Z baijum $
 """
+try:
+    #Declare this a namespace package if pkg_resources is available.
+    import pkg_resources
+    pkg_resources.declare_namespace('zope.app')
+except ImportError:
+    pass
 
