@@ -13,7 +13,7 @@
 ##############################################################################
 """Bootstrap tests
 
-$Id: tests.py 67630 2006-04-27 00:54:03Z jim $
+$Id: tests.py 79933 2007-09-24 22:23:58Z rogerineichen $
 """
 import unittest
 import transaction
@@ -21,12 +21,12 @@ import transaction
 from ZODB.tests.util import DB
 from zope.testing import doctest
 from zope.traversing.api import traverse
+from zope.error.interfaces import IErrorReportingUtility
+from zope.error.error import ErrorReportingUtility
 
 from zope.app import zapi
 from zope.app.component import hooks
 from zope.app.component.testing import PlacefulSetup
-from zope.app.error.error import ErrorReportingUtility
-from zope.app.error.interfaces import IErrorReportingUtility
 from zope.app.folder import rootFolder, Folder
 from zope.app.folder.interfaces import IRootFolder
 from zope.app.publication.zopepublication import ZopePublication
@@ -54,6 +54,7 @@ class TestBootstrapSubscriber(PlacefulSetup, unittest.TestCase):
         self.db = DB()
 
     def tearDown(self):
+        transaction.abort()
         PlacefulSetup.tearDown(self)
         self.db.close()
 
@@ -121,16 +122,23 @@ class TestBootstrapSubscriber(PlacefulSetup, unittest.TestCase):
         for i in range(2):
             cx = self.db.open()
             utility = ensureUtility(root_folder, IErrorReportingUtility,
-                                 'ErrorReporting', ErrorReportingUtility,
-                                 'ErrorReporting', asObject=True)
+                                    'ErrorReporting', ErrorReportingUtility,
+                                    'ErrorReporting', asObject=True)
+            utility2 = ensureUtility(root_folder, IErrorReportingUtility,
+                                     'ErrorReporting2', ErrorReportingUtility,
+                                     'ErrorReporting2', asObject=True)
             if utility != None:
                 name = utility.__name__
+                name2 = utility2.__name__
             else:
                 name = None
+                name2 = None
             if i == 0:
                 self.assertEqual(name, 'ErrorReporting')
+                self.assertEqual(name2, 'ErrorReporting2')
             else:
                 self.assertEqual(name, None)
+                self.assertEqual(name2, None)
 
             root = cx.root()
             root_folder = root[ZopePublication.root_name]
@@ -140,6 +148,8 @@ class TestBootstrapSubscriber(PlacefulSetup, unittest.TestCase):
 
             self.assert_(IErrorReportingUtility.providedBy(
                 traverse(package, 'ErrorReporting')))
+            self.assert_(IErrorReportingUtility.providedBy(
+                traverse(package, 'ErrorReporting2')))
             transaction.commit()
 
         cx.close()
