@@ -13,13 +13,21 @@
 ##############################################################################
 """Local Component Architecture
 
-$Id$
+$Id: __init__.py 70826 2006-10-20 03:41:16Z baijum $
 """
 __docformat__ = "reStructuredText"
+
 import zope.component
+import zope.deprecation
 
 _marker = object()
 
+# BBB: Deprecated on 9/26/2006
+@zope.deprecation.deprecate('''This function has been deprecated and will go
+away in Zope 3.6. There is no replacement for this function, since it does not
+make sense in light of registry bases anymore. If you are using this function
+to lookup the next utility, consider using get/queryNextUtility. Otherwise, it
+is suggested to iterate through the list of bases of a registry manually.''')
 def getNextSiteManager(context):
     """Get the next site manager."""
     sm = queryNextSiteManager(context, _marker)
@@ -29,6 +37,12 @@ def getNextSiteManager(context):
     return sm
 
 
+# BBB: Deprecated on 9/26/2006
+@zope.deprecation.deprecate('''This function has been deprecated and will go
+away in Zope 3.6. There is no replacement for this function, since it does not
+make sense in light of registry bases anymore. If you are using this function
+to lookup the next utility, consider using get/queryNextUtility. Otherwise, it
+is suggested to iterate through the list of bases of a registry manually.''')
 def queryNextSiteManager(context, default=None):
     """Get the next site manager.
 
