@@ -13,13 +13,19 @@
 ##############################################################################
 """Registration functional tests
 
-$Id: tests.py 67630 2006-04-27 00:54:03Z jim $
+$Id: tests.py 80836 2007-10-11 06:58:00Z srichter $
 """
-
+import os.path
 import unittest
 import zope.app.testing.functional
-
 from zope import interface
+from zope.app.component.testing import AppComponentLayer
+from zope.app.testing import functional
+from zope.testing import doctest
+
+AppComponentBrowserLayer = functional.ZCMLLayer(
+    os.path.join(os.path.dirname(__file__), 'ftesting.zcml'),
+    __name__, 'AppComponentBrowserLayer', allow_teardown=True)
 
 class ISampleBase(interface.Interface):
     pass
@@ -32,9 +38,16 @@ class Sample:
 
 
 def test_suite():
-    return zope.app.testing.functional.FunctionalDocFileSuite(
-        'registration.txt')
-        
+    site = zope.app.testing.functional.FunctionalDocFileSuite(
+        "site.txt",
+        optionflags=doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
+    site.layer = AppComponentBrowserLayer
+    reg = zope.app.testing.functional.FunctionalDocFileSuite(
+        'registration.txt',
+        optionflags=doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE)
+    reg.layer = AppComponentLayer
+    return unittest.TestSuite((site, reg))
+
 if __name__ == '__main__':
     unittest.main(defaultTest='test_suite')
 
