@@ -13,7 +13,7 @@
 ##############################################################################
 """Component Directives Tests
 
-$Id: test_directives.py 67630 2006-04-27 00:54:03Z jim $
+$Id: test_directives.py 73635 2007-03-26 15:47:22Z dobe $
 """
 import re
 import unittest
@@ -21,29 +21,26 @@ import pprint
 import warnings
 from cStringIO import StringIO
 
-from zope.interface import Interface, implements
+from zope.interface import implements
 from zope.testing.doctestunit import DocTestSuite
 from zope.component import createObject
 from zope.component.interfaces import IDefaultViewName
-from zope.component.registry import SubscriptionRegistration
 from zope.component.interfaces import ComponentLookupError
 from zope.component.interface import queryInterface
 
 from zope.configuration.xmlconfig import xmlconfig, XMLConfig
 from zope.configuration.exceptions import ConfigurationError
 from zope.configuration.xmlconfig import ZopeXMLConfigurationError
-from zope.security.proxy import removeSecurityProxy, getTestProxyItems
 from zope.security.checker import ProxyFactory, selectChecker
 
 import zope.app.component
 from zope.app import zapi
 from zope.app.testing.placelesssetup import PlacelessSetup
-from zope.app.component.tests.adapter import A1, A2, A3, Handler
+from zope.app.component.tests.adapter import A1, A2, A3
 from zope.app.component.tests.adapter import I1, I2, I3, IS
-from zope.app.component.tests.components import IContent, Content, Comp, comp
+from zope.app.component.tests.components import IContent, Content
 from zope.app.component.tests.components import IApp
 from zope.app.component.tests.views import Request, IV, IC, V1, R1, IR
-from zope.app.content.interfaces import IContentType
 
 from zope.app.component.tests import module, exampleclass
 
