@@ -15,18 +15,18 @@
 
 This vocabulary provides terms for all utilities providing a given interface.
 
-$Id: vocabulary.py 67630 2006-04-27 00:54:03Z jim $
+$Id: vocabulary.py 74311 2007-04-21 09:32:39Z ctheune $
 """
 __docformat__ = "reStructuredText"
 
 import zope.component
 from zope.interface import implements, classProvides, Interface
 from zope.interface.interfaces import IInterface
-from zope.interface.verify import verifyObject
-from zope.schema.interfaces import IVocabulary, IVocabularyTokenized
-from zope.schema.interfaces import ITokenizedTerm, IVocabularyFactory
+from zope.schema.interfaces import IVocabularyTokenized
+from zope.schema.interfaces import ITokenizedTerm, ITitledTokenizedTerm
+from zope.schema.interfaces import IVocabularyFactory
 
-from zope.app.i18n import ZopeMessageFactory as _
+from zope.app.component.i18n import ZopeMessageFactory as _
 from zope.app.interface.vocabulary import ObjectInterfacesVocabulary
 from zope.component.interfaces import IUtilityRegistration
 
@@ -34,8 +34,11 @@ class UtilityTerm(object):
     """A term representing a utility.
 
     The token of the term is the name of the utility. Here is a brief example
-    on how the IVocabulary interface is handled in this term as a utility:
-
+    on how the IVocabulary interface is handled in this term as a
+    utility:
+    
+    >>> from zope.interface.verify import verifyObject
+    >>> from zope.schema.interfaces import IVocabulary
     >>> term = UtilityTerm(IVocabulary, 'zope.schema.interfaces.IVocabulary')
     >>> verifyObject(ITokenizedTerm, term)
     True
@@ -250,35 +253,37 @@ class UtilityNameTerm:
     u'abc'
     >>> t2.value
     u'\xc0\xdf\xc7'
-    >>> t1.title()
+    >>> t1.title
     u'abc'
-    >>> repr(t2.title())
+    >>> repr(t2.title)
     "u'\\xc0\\xdf\\xc7'"
+    >>> ITitledTokenizedTerm.providedBy(t1)
+    True
 
     The tokens used for form values are Base-64 encodings of the
     names, with the letter 't' prepended to ensure the unnamed utility
     is supported:
 
-    >>> t1.token()
+    >>> t1.token
     'tYWJj'
-    >>> t2.token()
+    >>> t2.token
     'tw4DDn8OH'
 
-
     The unnamed utility is given an artificial title for use in user
     interfaces:
 
     >>> t3 = UtilityNameTerm(u'')
-    >>> t3.title()
+    >>> t3.title
     u'(unnamed utility)'
 
     """
 
-    implements(ITokenizedTerm)
+    implements(ITitledTokenizedTerm)
 
     def __init__(self, value):
         self.value = unicode(value)
 
+    @property
     def token(self):
         # Return our value as a token.  This is required to be 7-bit
         # printable ascii. We'll use base64 generated from the UTF-8
@@ -286,6 +291,7 @@ class UtilityNameTerm:
         # allowed to apply.)
         return "t" + self.value.encode('utf-8').encode('base64')[:-1]
 
+    @property
     def title(self):
         return self.value or _("(unnamed utility)")
 
@@ -301,6 +307,7 @@ class UtilityNames:
 
     >>> vocab = UtilityNames(IMyUtility)
 
+    >>> from zope.schema.interfaces import IVocabulary
     >>> IVocabulary.providedBy(vocab)
     True
     >>> IVocabularyTokenized.providedBy(vocab)
@@ -333,9 +340,15 @@ class UtilityNames:
     >>> u'' in vocab
     True
     >>> term1 = vocab.getTerm(u'')
-    >>> term2 = vocab.getTermByToken(term1.token())
+    >>> term2 = vocab.getTermByToken(term1.token)
     >>> term2.value
     u''
+    >>> term3 = vocab.getTerm(u'one')
+    >>> term3.token
+    'tb25l'
+    >>> term3a = vocab.getTermByToken('tb25l')
+    >>> term3.value
+    u'one'
 
     >>> placelesssetup.tearDown()
     """
@@ -359,7 +372,7 @@ class UtilityNames:
             if token == "t":
                 if not name:
                     break
-            elif name.encode('utf-8').encode('base64')[:-1] == token:
+            elif UtilityNameTerm(name).token == token:
                 break
         else:
             raise LookupError("no matching token: %r" % token)
