@@ -13,28 +13,28 @@
 ##############################################################################
 """Adding implementation tests
 
-$Id: test_adding.py 29457 2005-03-14 01:07:34Z srichter $
+$Id: test_adding.py 87422 2008-06-16 08:59:45Z ccomb $
 """
 import unittest
 
 import zope.interface
 import zope.security.checker
 from zope.component.interfaces import IFactory
-from zope.component.exceptions import ComponentLookupError
+from zope.component.interfaces import ComponentLookupError
 from zope.interface import implements, Interface, directlyProvides
 from zope.publisher.browser import TestRequest
 from zope.publisher.interfaces.browser import IBrowserRequest
+from zope.publisher.browser import BrowserView
 from zope.security.interfaces import ForbiddenAttribute
 from zope.testing.doctestunit import DocTestSuite
+from zope.exceptions.interfaces import UserError
+from zope.traversing.browser import AbsoluteURL
+from zope.traversing.browser.interfaces import IAbsoluteURL
+from zope.traversing.interfaces import IContainmentRoot
 
 from zope.app import zapi
 from zope.app.testing import ztapi
 from zope.app.testing.placelesssetup import PlacelessSetup, setUp, tearDown
-from zope.app.traversing.browser import AbsoluteURL
-from zope.app.traversing.browser.interfaces import IAbsoluteURL
-from zope.app.traversing.interfaces import IContainmentRoot
-from zope.app.exception.interfaces import UserError
-from zope.app.publisher.browser import BrowserView
 from zope.app.publisher.interfaces.browser import AddMenu
 from zope.app.publisher.interfaces.browser import IMenuItemType, IBrowserMenu
 from zope.app.publisher.browser.menu import BrowserMenuItem, BrowserMenu
@@ -47,6 +47,7 @@ from zope.app.container.contained import contained
 from zope.app.container.browser.adding import Adding
 from zope.app.container.sample import SampleContainer
 
+
 class Root(object):
     implements(IContainmentRoot)
 
@@ -139,7 +140,7 @@ class Test(PlacelessSetup, unittest.TestCase):
         ztapi.provideUtility(IFactory, Factory(), 'fooprivate')
 
         factory = Factory()
-        factory.__Security_checker__ = zope.security.checker.NamesChecker(      
+        factory.__Security_checker__ = zope.security.checker.NamesChecker(
             ['__call__'])
         ztapi.provideUtility(IFactory, factory, 'foo')
 
@@ -149,7 +150,7 @@ class Test(PlacelessSetup, unittest.TestCase):
         adding.nameAllowed = lambda: True
 
         # we can't use a private factory:
-        self.assertRaises(ForbiddenAttribute, 
+        self.assertRaises(ForbiddenAttribute,
                           adding.action, type_name='fooprivate', id='bar')
 
         # typical add - id is provided by user
@@ -163,7 +164,7 @@ class Test(PlacelessSetup, unittest.TestCase):
         self.assertRaises(UserError, adding.action, type_name='foo')
 
         # bad type_name
-        self.assertRaises(ComponentLookupError, adding.action, 
+        self.assertRaises(ComponentLookupError, adding.action,
             type_name='***', id='bar')
 
         # alternative add - id is provided internally instead of from user
@@ -182,7 +183,7 @@ class Test(PlacelessSetup, unittest.TestCase):
         adding.contentName = None
         adding.action(type_name='foo')
         self.assert_('Content' in container)
-        
+
 
     def test_action(self):
         container = Container()
@@ -195,10 +196,10 @@ class Test(PlacelessSetup, unittest.TestCase):
         ztapi.browserView(None, '', AbsoluteURL, providing=IAbsoluteURL)
         self.assertRaises(UserError, adding.action, '', 'foo')
         adding.action('Thing', 'foo')
-        self.assertEqual(adding.request.response._headers['location'],
+        self.assertEqual(adding.request.response.getHeader('location'),
                          '/container/+/Thing=foo')
         adding.action('Thing/screen1', 'foo')
-        self.assertEqual(adding.request.response._headers['location'],
+        self.assertEqual(adding.request.response.getHeader('location'),
                          '/container/+/Thing/screen1=foo')
 
     def test_publishTraverse_factory(self):
@@ -212,7 +213,6 @@ class Test(PlacelessSetup, unittest.TestCase):
 
 def test_constraint_driven_addingInfo():
     """
-    >>> setUp()
     >>> registerAddMenu()
 
     >>> class TestMenu(zope.interface.Interface):
@@ -224,7 +224,7 @@ def test_constraint_driven_addingInfo():
     ...                      'TestMenu')
 
     >>> defineMenuItem(TestMenu, IAdding, '', 'item1')
-    >>> defineMenuItem(TestMenu, IAdding, '', 'item2')
+    >>> defineMenuItem(TestMenu, IAdding, '', 'Item2')
 
     >>> defineMenuItem(AddMenu, IAdding, '', 'item3', extra={'factory': 'f1'})
     >>> defineMenuItem(AddMenu, IAdding, '', 'item4', extra={'factory': 'f2'})
@@ -263,24 +263,22 @@ def test_constraint_driven_addingInfo():
     >>> len(items)
     1
     >>> items[0]['title']
-    'item3'
-    
+    u'item3'
+
     >>> adding.menu_id = 'TestMenu'
     >>> items = adding.addingInfo()
     >>> len(items)
     3
     >>> items[0]['title']
-    'item1'
-    >>> items[1]['title']
-    'item2'
+    u'item1'
+    >>> items[1]['title'] # the collator ordered this one correctly!
+    u'Item2'
     >>> items[2]['title']
-    'item3'
-    >>> tearDown()    
+    u'item3'
     """
 
 def test_constraint_driven_add():
     """
-    >>> setUp()
     >>> from zope.app.container.sample import SampleContainer
     >>> from zope.app.container.browser.adding import Adding
 
@@ -294,7 +292,7 @@ def test_constraint_driven_add():
     ...     "a mock item constraint "
     ...     if not isinstance(object, F1):
     ...         raise zope.interface.Invalid('not a valid child')
-    
+
     >>> class ITestContainer(zope.interface.Interface):
     ...     def __setitem__(name, object):
     ...         pass
@@ -305,10 +303,10 @@ def test_constraint_driven_add():
 
     >>> adding = Adding(Container(), TestRequest())
     >>> c = adding.add(F1())
-    
+
     This test should fail, because the container only
     accepts instances of F1
-    
+
     >>> adding.add(F2())
     Traceback (most recent call last):
     ...
@@ -329,7 +327,7 @@ def test_constraint_driven_add():
     >>> zope.interface.classImplements(F1, I2)
 
     This adding now fails, because the Container is not a valid
-    parent for F1 
+    parent for F1
 
     >>> c = adding.add(F1())
     Traceback (most recent call last):
@@ -339,32 +337,30 @@ def test_constraint_driven_add():
     >>> adding = Adding(ValidContainer(), TestRequest())
     >>> c = adding.add(F1())
 
-    >>> tearDown()
     """
 
 
 def test_nameAllowed():
     """
-    Test for nameAllowed in adding.py 
-    
-    >>> setUp()
+    Test for nameAllowed in adding.py
+
     >>> from zope.app.container.browser.adding import Adding
     >>> from zope.app.container.interfaces import IContainerNamesContainer
 
     Class implements IContainerNamesContainer
-    
+
     >>> class FakeContainer(object):
     ...    zope.interface.implements(IContainerNamesContainer)
 
     nameAllowed returns False if the class imlements
     IContainerNamesContainer
-    
+
     >>> adding = Adding(FakeContainer(),TestRequest())
     >>> adding.nameAllowed()
     False
 
     Fake class without IContainerNamesContainer
-    
+
     >>> class Fake(object):
     ...    pass
 
@@ -381,13 +377,15 @@ def test_nameAllowed():
 
 def test_chooseName():
     """If user don't enter name, pick one
-    
+
     >>> class MyContainer(object):
+    ...    args = {}
     ...    zope.interface.implements(INameChooser, IContainer)
     ...    def chooseName(self, name, object):
+    ...        self.args["choose"] = name, object
     ...        return 'pickone'
     ...    def checkName(self, name, object):
-    ...        pass
+    ...        self.args["check"] = name, object
     ...    def __setitem__(self, name, object):
     ...        setattr(self, name, object)
     ...        self.name = name
@@ -403,16 +401,28 @@ def test_chooseName():
     'pickone'
     >>> add_obj is o
     True
+
+    Make sure right arguments passed to INameChooser adapter:
+
+    >>> name, obj = mycontainer.args["choose"]
+    >>> name
+    ''
+    >>> obj is o
+    True
+    >>> name, obj = mycontainer.args["check"]
+    >>> name
+    'pickone'
+    >>> obj is o
+    True
     """
-    
+
 
 
 def test_SingleMenuItem_and_CustomAddView_NonICNC():
     """
     This tests the condition if the content has Custom Add views and
     the container contains only a single content object
-    
-    >>> setUp()
+
     >>> registerAddMenu()
     >>> defineMenuItem(AddMenu, IAdding, '', 'item3', extra={'factory': 'f1'})
 
@@ -452,27 +462,25 @@ def test_SingleMenuItem_and_CustomAddView_NonICNC():
 
     isSingleMenuItem returns True if there is only one content class
     inside the Container
-    
+
     >>> adding.isSingleMenuItem()
     True
 
     hasCustomAddView will return False as the content does not have
     a custom Add View
-    
+
     >>> adding.hasCustomAddView()
     True
-    
-    >>> tearDown()    
+
     """
 
 def test_SingleMenuItem_and_NoCustomAddView_NonICNC():
     """
-    
+
     This function checks the case where there is a single content object
     and there is non custom add view . Also the container does not
     implement IContainerNamesContainer
-    
-    >>> setUp()
+
     >>> registerAddMenu()
     >>> defineMenuItem(AddMenu, None, '', 'item3', extra={'factory': ''})
     >>> class F1(object):
@@ -502,7 +510,7 @@ def test_SingleMenuItem_and_NoCustomAddView_NonICNC():
     >>> from zope.component.factory import Factory
     >>> ztapi.provideUtility(IFactory, Factory(F1), 'f1')
     >>> ztapi.provideUtility(IFactory, Factory(F2), 'f2')
-    
+
     >>> from zope.app.container.browser.adding import Adding
     >>> adding = Adding(Container(), TestRequest())
     >>> items = adding.addingInfo()
@@ -511,7 +519,7 @@ def test_SingleMenuItem_and_NoCustomAddView_NonICNC():
 
     The isSingleMenuItem will return True if there is one single content
     that can be added inside the Container
-    
+
     >>> adding.isSingleMenuItem()
     True
 
@@ -520,8 +528,7 @@ def test_SingleMenuItem_and_NoCustomAddView_NonICNC():
 
     >>> adding.hasCustomAddView()
     False
-    
-    >>> tearDown()
+
     """
 
 def test_isSingleMenuItem_with_ICNC():
@@ -529,10 +536,9 @@ def test_isSingleMenuItem_with_ICNC():
     This test checks for whether there is a single content that can be added
     and the container uses IContainerNamesContaienr
 
-    >>> setUp()
     >>> registerAddMenu()
     >>> defineMenuItem(AddMenu, None, '', 'item3', extra={'factory': ''})
-    
+
     >>> class F1(object):
     ...     pass
 
@@ -566,17 +572,14 @@ def test_isSingleMenuItem_with_ICNC():
     True
     >>> adding.hasCustomAddView()
     False
-    
-    >>> tearDown()
-    
+
     """
 
 def test_suite():
     return unittest.TestSuite((
         unittest.makeSuite(Test),
-        DocTestSuite(),
+        DocTestSuite(setUp=setUp, tearDown=tearDown),
         ))
 
 if __name__=='__main__':
     unittest.main(defaultTest='test_suite')
-
