@@ -13,10 +13,11 @@
 ##############################################################################
 """Browser widgets with text-based input
 
-$Id: textwidgets.py 70836 2006-10-20 07:30:46Z baijum $
+$Id: textwidgets.py 77142 2007-06-27 14:37:09Z zagy $
 """
 __docformat__ = 'restructuredtext'
 
+import decimal
 from xml.sax import saxutils
 from zope.interface import implements
 from zope.datetime import parseDatetimetz
@@ -24,12 +25,11 @@ from zope.datetime import DateTimeError
 from zope.i18n.format import DateTimeParseError
 
 from zope.app.form.interfaces import IInputWidget, ConversionError
+from zope.app.form.browser.i18n import _
 from zope.app.form.browser.interfaces import ITextBrowserWidget
 from zope.app.form.browser.widget import SimpleInputWidget, renderElement
 from zope.app.form.browser.widget import DisplayWidget
 
-from zope.app.i18n import ZopeMessageFactory as _
-
 
 def escape(str):
     if str is not None:
@@ -286,6 +286,48 @@ class TextAreaWidget(SimpleInputWidget):
       name="field.foo"
       rows="15"
       >&lt;h1&gt;&amp;copy;&lt;/h1&gt;</textarea>
+
+    There was a but which caused the content of <textarea> tags not to be
+    rendered correctly when there was a conversion error. Make sure the quoting
+    works correctly::
+
+    >>> from zope.schema import Text
+    >>> field = Text(__name__='description', title=u'Description')
+
+    >>> from zope.app.form.interfaces import ConversionError
+    >>> class TestTextAreaWidget(TextAreaWidget):
+    ...     def _toFieldValue(self, input):
+    ...         if 'foo' in input:
+    ...             raise ConversionError("I don't like foo.")
+    ...         return input
+    ...
+
+    >>> request = TestRequest(form={'field.description': u'<p>bar</p>'})
+    >>> widget = TestTextAreaWidget(field, request)
+    >>> widget.getInputValue()
+    u'<p>bar</p>'
+    >>> print normalize( widget() )
+    <textarea
+      cols="60"
+      id="field.description"
+      name="field.description"
+      rows="15"
+      >&lt;p&gt;bar&lt;/p&gt;</textarea>
+
+    >>> request = TestRequest(form={'field.description': u'<p>foo</p>'})
+    >>> widget = TestTextAreaWidget(field, request)
+    >>> try:
+    ...     widget.getInputValue()
+    ... except ConversionError, error:
+    ...     print error.doc()
+    I don't like foo.
+    >>> print normalize( widget() )
+    <textarea
+      cols="60"
+      id="field.description"
+      name="field.description"
+      rows="15"
+      >&lt;p&gt;foo&lt;/p&gt;</textarea>
     """
 
     default = ""
@@ -309,7 +351,6 @@ class TextAreaWidget(SimpleInputWidget):
         value = super(TextAreaWidget, self)._toFormValue(value)
         if value:
             value = value.replace("\n", "\r\n")
-            value = escape(value)
         else:
             value = u''
 
@@ -323,7 +364,7 @@ class TextAreaWidget(SimpleInputWidget):
                              rows=self.height,
                              cols=self.width,
                              style=self.style,
-                             contents=self._getFormValue(),
+                             contents=escape(self._getFormValue()),
                              extra=self.extra)
 
 class BytesAreaWidget(Bytes, TextAreaWidget):
@@ -387,10 +428,20 @@ class PasswordWidget(TextWidget):
                                  size=self.displayWidth,
                                  extra=self.extra)
 
+    def _toFieldValue(self, input):
+        try:
+            existing = self.context.get(self.context.context)
+        except AttributeError:
+            existing = False
+        if (not input) and existing:
+            return self.context.UNCHANGED_PASSWORD
+        return super(PasswordWidget, self)._toFieldValue(input)
+
     def hidden(self):
         raise NotImplementedError(
             'Cannot get a hidden tag for a password field')
 
+
 class FileWidget(TextWidget):
     """File Widget"""
 
@@ -484,6 +535,26 @@ class FloatWidget(TextWidget):
             except ValueError, v:
                 raise ConversionError(_("Invalid floating point data"), v)
 
+class DecimalWidget(TextWidget):
+    implements(IInputWidget)
+    displayWidth = 10
+
+    def _toFieldValue(self, input):
+        if input == self._missing:
+            return self.context.missing_value
+        else:
+            try:
+                return decimal.Decimal(input)
+            except decimal.InvalidOperation, v:
+                raise ConversionError(_("Invalid decimal data"), v)
+
+    def _toFormValue(self, value):
+        if value == self.context.missing_value:
+            value = self._missing
+        else:
+            return unicode(value)
+
+
 class DatetimeWidget(TextWidget):
     """Datetime entry widget."""
 
