@@ -13,7 +13,7 @@
 ##############################################################################
 """RAM cache implementation.
 
-$Id: ram.py 41602 2006-02-12 12:32:20Z hdima $
+$Id: ram.py 77128 2007-06-27 13:32:02Z nouri $
 """
 __docformat__ = 'restructuredtext'
 
@@ -286,6 +286,7 @@ class Storage(object):
         "Cleanup the data"
         self.removeStaleEntries()
         self.removeLeastAccessed()
+        self.lastCleanup = time()
 
     def removeLeastAccessed(self):
         ""
@@ -298,7 +299,7 @@ class Storage(object):
             if len(keys) > self.maxEntries:
                 def getKey(item):
                     ob, key = item
-                    return data[ob][key]
+                    return data[ob][key][2]
                 keys.sort(key=getKey)
 
                 ob, key = keys[self.maxEntries]
@@ -338,7 +339,7 @@ class Storage(object):
             hits = sum(entry[2] for entry in self._data[ob].itervalues())
             result.append({'path': ob,
                            'hits': hits,
-                           'misses': self._misses[ob],
+                           'misses': self._misses.get(ob, 0),
                            'size': size,
                            'entries': len(self._data[ob])})
         return tuple(result)
