@@ -13,19 +13,19 @@
 ##############################################################################
 """Unit tests for RAM Cache.
 
-$Id: test_ramcache.py 29143 2005-02-14 22:43:16Z srichter $
+$Id: test_ramcache.py 77128 2007-06-27 13:32:02Z nouri $
 """
 from time import time
 from unittest import TestCase, TestSuite, main, makeSuite
 
 from zope.interface.verify import verifyClass, verifyObject
 from zope.interface import implements
+from zope.traversing.interfaces import IPhysicallyLocatable
 
 from zope.app.cache.ram import RAMCache
 from zope.app.cache.tests.test_icache import BaseICacheTest
 from zope.app.cache.interfaces import ICache
 from zope.app.cache.interfaces.ram import IRAMCache
-from zope.app.traversing.interfaces import IPhysicallyLocatable
 from zope.app.testing.placelesssetup import PlacelessSetup
 
 class Locatable(object):
@@ -87,6 +87,17 @@ class TestRAMCache(PlacelessSetup,
         self.assertEqual(s.maxAge, 2, "maxAge not set")
         self.assertEqual(s.cleanupInterval, 3, "cleanupInterval not set")
 
+    def test_timedCleanup(self):
+        from zope.app.cache.ram import RAMCache
+        import time
+        c = RAMCache()
+        c.update(cleanupInterval=1, maxAge=2)
+        lastCleanup = c._getStorage().lastCleanup
+        time.sleep(2)
+        c.set(42, "object", key={'foo': 'bar'})
+        # last cleanup should now be updated
+        self.failUnless(lastCleanup < c._getStorage().lastCleanup)
+
     def test_cache(self):
         from zope.app.cache import ram
         self.assertEqual(type(ram.caches), type({}),
@@ -238,6 +249,21 @@ class TestStorage(TestCase):
             raise "ExpectedKeyError"
         self.assertEqual(s._misses[object2], 1)
 
+    def test_getEntry_do_cleanup(self):
+         from zope.app.cache.ram import Storage
+
+         s = Storage(cleanupInterval=300, maxAge=300)
+         object = 'object'
+         key = ('view', (), ('answer', 42))
+         value = 'yes'
+
+         s.setEntry(object, key, value)
+
+         s._data[object][key][1] = time() - 400
+         s.lastCleanup = time() - 400
+
+         self.assertRaises(KeyError, s.getEntry, object, key)
+
     def test_setEntry(self):
         from zope.app.cache.ram import Storage
 
@@ -434,15 +460,15 @@ class TestStorage(TestCase):
         value = 'yes'
         timestamp = time()
         s._data = {object:  {key1: [value, 1, 10],
-                             key2: [value, 2, 5],
-                             key3: [value, 3, 2]},
-                   object2: {key1: [value, 4, 2],
-                             key2: [value, 5, 1],
-                             key3: [value, 6, 1]}}
+                             key2: [value, 6, 5],
+                             key3: [value, 2, 2]},
+                   object2: {key1: [value, 5, 2],
+                             key2: [value, 3, 1],
+                             key3: [value, 4, 1]}}
         s.removeLeastAccessed()
         self.assertEqual(s._data,
                          {object:  {key1: [value, 1, 0],
-                                    key2: [value, 2, 0]}},
+                                    key2: [value, 6, 0]}},
                          'least records removed incorrectly')
 
         s = Storage(maxEntries=6)
@@ -487,8 +513,8 @@ class TestStorage(TestCase):
         self.assertEqual(s._misses, clearMisses, "misses counter not cleared")
 
     def test_getStatistics(self):
-        from zope.app.cache.ram import Storage
-        from pickle import dumps
+        from zope.app.cache.ram import Storage, dumps
+
         s = Storage(maxEntries=3)
         object = 'object'
         object2 = 'object2'
@@ -523,7 +549,7 @@ class TestStorage(TestCase):
 
         result = s.getStatistics()
 
-        self.assertEqual(result, expected, "got unexpected stats")
+        self.assertEqual(result, expected)
 
 
 class TestModule(TestCase):
