@@ -13,7 +13,7 @@
 ##############################################################################
 """Catalog Interfaces
 
-$Id: interfaces.py 39064 2005-10-11 18:40:10Z philikon $
+$Id: interfaces.py 80209 2007-09-27 09:38:31Z berndroessl $
 """
 
 import zope.index.interfaces
@@ -22,7 +22,7 @@ import zope.schema
 import zope.app.container.interfaces
 import zope.app.container.constraints
 
-from zope.app.i18n import ZopeMessageFactory as _
+from zope.i18nmessageid import ZopeMessageFactory as _
 
 class ICatalogQuery(zope.interface.Interface):
     """Provides Catalog Queries."""
@@ -77,4 +77,12 @@ class IAttributeIndex(zope.interface.Interface):
         description=_(u"If true, then the field should be called to get the "
                       u"value to be indexed"),
         )
-        
+
+
+class INoAutoIndex(zope.interface.Interface):
+
+    """Marker for objects that should not be automatically indexed"""
+
+class INoAutoReindex(zope.interface.Interface):
+
+    """Marker for objects that should not be automatically reindexed"""
