@@ -16,25 +16,22 @@
 Note that indexes &c already have test suites, we only have to check that
 a catalog passes on events that it receives.
 
-$Id: tests.py 71191 2006-11-20 09:40:24Z adamg $
+$Id: tests.py 79790 2007-09-21 10:37:23Z berndroessl $
 """
 import unittest
 from zope.testing import doctest
-
-import BTrees.IFBTree
-
+from zope import component
 from zope.interface import implements
 from zope.interface.verify import verifyObject
 from zope.app.testing import ztapi, setup, placelesssetup
 from BTrees.IFBTree import IFSet
 from zope.app.intid.interfaces import IIntIds
 from zope.app.component.hooks import setSite
+from zope.location.location import Location
 
 from zope.index.interfaces import IInjection, IIndexSearch
 from zope.app.catalog.interfaces import ICatalog
 from zope.app.catalog.catalog import Catalog
-from zope.app import zapi
-from zope.app.catalog.attribute import AttributeIndex
 from zope.app.catalog.field import FieldIndex
 
 class ReferenceStub:
@@ -218,11 +215,8 @@ class CatalogStub:
     def unindex_doc(self, docid):
         self.unregs.append(docid)
 
-class Stub:
-
-    __name__ = None
-    __parent__ = None
-
+class Stub(Location):
+    pass
 
 
 class TestEventSubscribers(unittest.TestCase):
@@ -245,6 +239,9 @@ class TestEventSubscribers(unittest.TestCase):
         ob = Stub()
         ob2 = Stub()
 
+        self.root['ob'] = ob
+        self.root['ob2'] = ob2
+
         id = self.utility.register(ob)
         indexDocSubscriber(IntIdAddedEvent(ob, ObjectAddedEvent(ob2)))
 
@@ -256,6 +253,8 @@ class TestEventSubscribers(unittest.TestCase):
         from zope.lifecycleevent import ObjectModifiedEvent
 
         ob = Stub()
+        self.root['ob'] = ob
+
         id = self.utility.register(ob)
 
         reindexDocSubscriber(ObjectModifiedEvent(ob))
@@ -264,10 +263,12 @@ class TestEventSubscribers(unittest.TestCase):
         self.assertEqual(self.cat.unregs, [])
 
         ob2 = Stub()
+        self.root['ob2'] = ob2
+
         reindexDocSubscriber(ObjectModifiedEvent(ob2))
         self.assertEqual(self.cat.regs, [(1, ob)])
         self.assertEqual(self.cat.unregs, [])
-        
+
 
     def test_unindexDocSubscriber(self):
         from zope.app.catalog.catalog import unindexDocSubscriber
@@ -277,6 +278,10 @@ class TestEventSubscribers(unittest.TestCase):
         ob = Stub()
         ob2 = Stub()
         ob3 = Stub()
+        self.root['ob'] = ob
+        self.root['ob2'] = ob2
+        self.root['ob3'] = ob3
+
         id = self.utility.register(ob)
 
         unindexDocSubscriber(
@@ -291,33 +296,33 @@ class TestEventSubscribers(unittest.TestCase):
 
 
 class TestIndexUpdating(unittest.TestCase) :
-    """Issue #466: When reindexing a catalog it takes all objects from 
-    the nearest IntId utility. This is a problem when IntId utility 
+    """Issue #466: When reindexing a catalog it takes all objects from
+    the nearest IntId utility. This is a problem when IntId utility
     lives in another site than the one.
-            
+
     To solve this issue we simply check whether the objects are living
     within the nearest site.
     """
 
     def setUp(self):
-        
+
         setup.placefulSetUp(True)
-        
+
         from zope.app.catalog.catalog import Catalog
         from zope.app.container.contained import ContainerSublocations
-        
+
         self.root = setup.buildSampleFolderTree()
-        
+
         subfolder = self.root[u'folder1'][u'folder1_1']
         root_sm = self.root_sm = setup.createSiteManager(self.root)
         local_sm = self.local_sm = setup.createSiteManager(subfolder)
         self.utility = setup.addUtility(root_sm, '', IIntIds, IntIdsStub())
         self.cat = setup.addUtility(local_sm, '', ICatalog, Catalog())
         self.cat['name'] = StubIndex('__name__', None)
-        
+
         for obj in self.iterAll(self.root) :
             self.utility.register(obj)
-      
+
     def tearDown(self):
         setup.placefulTearDown()
 
@@ -328,56 +333,266 @@ class TestIndexUpdating(unittest.TestCase) :
             if IContainer.providedBy(value) :
                 for obj in self.iterAll(value) :
                     yield obj
-                    
+
     def test_visitSublocations(self) :
         """ Test the iterContained method which should return only the
         sublocations which are registered by the IntIds.
         """
-        
+
         names = sorted([ob.__name__ for i, ob in self.cat._visitSublocations()])
         self.assertEqual(names, [u'folder1_1', u'folder1_1_1', u'folder1_1_2'])
-            
+
     def test_updateIndex(self):
         """ Setup a catalog deeper within the containment hierarchy
         and call the updateIndexes method. The indexed objects should should
         be restricted to the sublocations.
         """
-        
+
         self.cat.updateIndexes()
         index = self.cat['name']
         names = sorted([ob.__name__ for i, ob in index.doc.items()])
         self.assertEqual(names, [u'folder1_1', u'folder1_1_1', u'folder1_1_2'])
-        
-class TestCatalogBugs(unittest.TestCase):
+
+    def test_optimizedUpdateIndex(self):
+        """ Setup a catalog deeper within the containment hierarchy together
+        with its intid utility. The catalog will not visit sublocations
+        because the intid utility can not contain objects outside the site
+        where it is registered.
+        """
+        utility = setup.addUtility(self.local_sm, '', IIntIds, IntIdsStub())
+        subfolder = self.root[u'folder1'][u'folder1_1']
+        for obj in self.iterAll(subfolder) :
+            utility.register(obj)
+
+        self.cat.updateIndexes()
+        index = self.cat['name']
+        names = sorted([ob.__name__ for i, ob in index.doc.items()])
+        self.assertEqual(names, [u'folder1_1_1', u'folder1_1_2'])
+
+
+class TestSubSiteCatalog(unittest.TestCase) :
+    """If a catalog is defined in a sub site and the hooks.setSite was
+    not set the catalog will not be found unless the context in
+    getAllUtilitiesRegisteredFor is set.
+    """
+
+    def setUp(self):
+
+        setup.placefulSetUp(True)
+
+        from zope.app.catalog.catalog import Catalog
+        from zope.app.container.contained import ContainerSublocations
+
+        self.root = setup.buildSampleFolderTree()
+
+        self.subfolder = self.root[u'folder1'][u'folder1_1']
+        root_sm = self.root_sm = setup.createSiteManager(self.root)
+        local_sm = self.local_sm = setup.createSiteManager(self.subfolder)
+        self.utility = setup.addUtility(root_sm, '', IIntIds, IntIdsStub())
+        self.cat = setup.addUtility(local_sm, '', ICatalog, Catalog())
+        self.cat['name'] = StubIndex('__name__', None)
+
+        for obj in self.iterAll(self.root) :
+            self.utility.register(obj)
+
+
+    def tearDown(self):
+        setup.placefulTearDown()
+
+    def iterAll(self, container) :
+        from zope.app.container.interfaces import IContainer
+        for value in container.values() :
+            yield value
+            if IContainer.providedBy(value) :
+                for obj in self.iterAll(value) :
+                    yield obj
+
+
+
+    def test_Index(self):
+        """ Setup a catalog deeper within the containment hierarchy
+        and call the updateIndexes method. The indexed objects should should
+        be restricted to the sublocations.
+        """
+        from zope.app.catalog.catalog import indexDocSubscriber
+        from zope.app.container.contained import ObjectAddedEvent
+
+        ob = Stub()
+        self.subfolder['ob'] = ob
+
+        id = self.utility.register(ob)
+
+        setSite(self.subfolder)
+        res = self.cat.searchResults(name='ob')
+        self.assertEqual(len(res), 0)
+
+        setSite(None)
+        indexDocSubscriber(ObjectAddedEvent(ob))
+
+        setSite(self.subfolder)
+        res = self.cat.searchResults(name='ob')
+        self.assertEqual(len(res), 1)
+
+
+    def test_updateIndex(self):
+        """ Setup a catalog deeper within the containment hierarchy
+        and call the updateIndexes method. The indexed objects should should
+        be restricted to the sublocations.
+        """
+        from zope.app.catalog.catalog import reindexDocSubscriber
+        from zope.lifecycleevent import ObjectModifiedEvent
+
+        ob = Stub()
+        self.subfolder['ob'] = ob
+
+        id = self.utility.register(ob)
+
+        setSite(self.subfolder)
+        res = self.cat.searchResults(name='ob')
+        self.assertEqual(len(res), 0)
+
+        setSite(None)
+        reindexDocSubscriber(ObjectModifiedEvent(ob))
+
+        setSite(self.subfolder)
+        res = self.cat.searchResults(name='ob')
+        self.assertEqual(len(res), 1)
+
+    def test_UnIndex(self):
+        """ Setup a catalog deeper within the containment hierarchy
+        and call the updateIndexes method. The indexed objects should should
+        be restricted to the sublocations.
+        """
+        from zope.app.catalog.catalog import indexDocSubscriber
+        from zope.app.container.contained import ObjectAddedEvent
+        from zope.app.catalog.catalog import unindexDocSubscriber
+        from zope.app.container.contained import ObjectRemovedEvent
+
+        ob = Stub()
+        self.subfolder['ob'] = ob
+
+        id = self.utility.register(ob)
+
+        setSite(self.subfolder)
+        res = self.cat.searchResults(name='ob')
+        self.assertEqual(len(res), 0)
+
+        setSite(None)
+        indexDocSubscriber(ObjectAddedEvent(ob))
+
+        setSite(self.subfolder)
+        res = self.cat.searchResults(name='ob')
+        self.assertEqual(len(res), 1)
+
+        setSite(None)
+        unindexDocSubscriber(ObjectRemovedEvent(ob))
+
+        setSite(self.subfolder)
+        res = self.cat.searchResults(name='ob')
+        self.assertEqual(len(res), 0)
+
+
+class TestCatalogBugs(placelesssetup.PlacelessSetup, unittest.TestCase):
     """I found that z.a.catalog, AttributeIndex failed to remove the previous
     value/object from the index IF the NEW value is None.
     """
-    
+
     def test_updateIndexWithNone(self):
         uidutil = IntIdsStub()
         ztapi.provideUtility(IIntIds, uidutil)
-        
+
         catalog = Catalog()
         index = FieldIndex('author', None)
         catalog['author'] = index
-        
+
         ob1 = stoopid(author = "joe")
         ob1id = uidutil.register(ob1)
         catalog.index_doc(ob1id, ob1)
-        
+
         res = catalog.searchResults(author=('joe','joe'))
         names = [x.author for x in res]
         names.sort()
         self.assertEqual(len(names), 1)
         self.assertEqual(names, ['joe'])
-        
+
         ob1.author = None
         catalog.index_doc(ob1id, ob1)
-        
+
+        #the index must be empty now because None values are never indexed
+        res = catalog.searchResults(author=(None, None))
+        self.assertEqual(len(res), 0)
+
+    def test_updateIndexFromCallableWithNone(self):
+        uidutil = IntIdsStub()
+        ztapi.provideUtility(IIntIds, uidutil)
+
+        catalog = Catalog()
+        index = FieldIndex('getAuthor', None, field_callable=True)
+        catalog['author'] = index
+
+        ob1 = stoopidCallable(author = "joe")
+
+        ob1id = uidutil.register(ob1)
+        catalog.index_doc(ob1id, ob1)
+
         res = catalog.searchResults(author=('joe','joe'))
         names = [x.author for x in res]
-        #joe must not be here anymore
-        self.assertEqual(len(names), 0)
+        names.sort()
+        self.assertEqual(len(names), 1)
+        self.assertEqual(names, ['joe'])
+
+        ob1.author = None
+        catalog.index_doc(ob1id, ob1)
+
+        #the index must be empty now because None values are never indexed
+        res = catalog.searchResults(author=(None, None))
+        self.assertEqual(len(res), 0)
+
+class stoopidCallable(object):
+    def __init__(self, **kw):
+        #leave the door open to not to set self.author
+        self.__dict__.update(kw)
+
+    def getAuthor(self):
+        return self.author
+
+class TestIndexRaisingValueGetter(placelesssetup.PlacelessSetup, unittest.TestCase):
+    """ """
+    def test_IndexRaisingValueGetter(self):
+        """We can have indexes whose values are determined by callable
+        methods.
+        Raising an exception in the method should not be silently ignored
+        That would cause index corruption -- the index would be out of sync"""
+        uidutil = IntIdsStub()
+        ztapi.provideUtility(IIntIds, uidutil)
+
+        catalog = Catalog()
+        index = FieldIndex('getAuthor', None, field_callable=True)
+        catalog['author'] = index
+
+        ob1 = stoopidCallable(author = "joe")
+        ob1id = uidutil.register(ob1)
+        catalog.index_doc(ob1id, ob1)
+
+        res = catalog.searchResults(author=('joe','joe'))
+        names = [x.author for x in res]
+        names.sort()
+        self.assertEqual(len(names), 1)
+        self.assertEqual(names, ['joe'])
+
+        ob2 = stoopidCallable() # no author here, will raise AttributeError
+        ob2id = uidutil.register(ob2)
+        try:
+            catalog.index_doc(ob2id, ob2)
+            self.fail("AttributeError exception should be raised")
+        except AttributeError:
+            #this is OK, we WANT to have the exception
+            pass
+
+
+def setUp(test):
+    root = setup.placefulSetUp(True)
+    test.globs['root'] = root
 
 def test_suite():
     from zope.testing import doctest
@@ -385,13 +600,22 @@ def test_suite():
     suite.addTest(unittest.makeSuite(Test))
     suite.addTest(unittest.makeSuite(TestEventSubscribers))
     suite.addTest(unittest.makeSuite(TestIndexUpdating))
+    suite.addTest(unittest.makeSuite(TestSubSiteCatalog))
     suite.addTest(unittest.makeSuite(TestCatalogBugs))
+    suite.addTest(unittest.makeSuite(TestIndexRaisingValueGetter))
     suite.addTest(doctest.DocTestSuite('zope.app.catalog.attribute'))
     suite.addTest(doctest.DocFileSuite(
         'README.txt',
         setUp=placelesssetup.setUp,
         tearDown=placelesssetup.tearDown,
         ))
+    suite.addTest(doctest.DocFileSuite(
+        'event.txt',
+        setUp=setUp,
+        tearDown=lambda x: setup.placefulTearDown(),
+        optionflags=doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS,
+        ))
+
     return suite
 
 
