@@ -13,7 +13,7 @@
 ##############################################################################
 """File views.
 
-$Id: file.py 67711 2006-04-28 11:01:55Z dobe $
+$Id: file.py 90902 2008-09-05 19:54:28Z nadako $
 """
 import zope.event
 from zope import lifecycleevent
@@ -24,9 +24,9 @@ from zope.exceptions.interfaces import UserError
 
 from zope.app.file.file import File
 from zope.app.file.interfaces import IFile
-from zope.app.i18n import ZopeMessageFactory as _
-from zope.dublincore.interfaces import IZopeDublinCore
-import zope.datetime 
+from zope.app.file.i18n import ZopeMessageFactory as _
+from zope.dublincore.interfaces import IDCTimes
+import zope.datetime
 
 import time
 from datetime import datetime
@@ -39,8 +39,8 @@ class FileView(object):
 
         """Sets various headers if the request is present and returns the
         data of the file. If the "If-Modified-Since" header is set and
-        the context implements IZopeDublinCore, data is only returned if
-        the modification date of the context is new than the date in the
+        the context is adaptable to IDCTimes, data is only returned if
+        the modification date of the context is newer than the date in the
         "If-Modified-Since" header
         >>> from zope.publisher.browser import BrowserView, TestRequest
         >>> class FileTestView(FileView, BrowserView): pass
@@ -62,37 +62,37 @@ class FileView(object):
         >>> request.response.getHeader('Content-Length')
         '12'
 
-        If the file is adaptable to IZopeDublinCore the
-        "Last-Modified" header is also set.
-        
+        If the file is adaptable to IDCTimes the "Last-Modified" header is also
+        set.
+
         >>> request.response.getHeader('Last-Modified') is None
         True
 
         For the test we just declare that our file provides
         IZopeDublinCore
+        >>> from zope.dublincore.interfaces import IZopeDublinCore
         >>> from zope.interface import directlyProvides
         >>> directlyProvides(aFile,IZopeDublinCore)
         >>> request = TestRequest()
         >>> view = FileTestView(aFile,request)
         >>> view.show()
         'data of file'
-        >>> datetime.fromtimestamp(zope.datetime.time(
-        ...     request.response.getHeader('Last-Modified')))
-        datetime.datetime(2006, 1, 1, 0, 0)
+        >>> request.response.getHeader('Last-Modified')
+        'Sun, 01 Jan 2006 00:00:00 GMT'
 
         If the "If-Modified-Since" header is set and is newer a 304
         status is returned and the value is empty.
-        
-        >>> modified = datetime.now()
-        >>> modHeader = zope.datetime.rfc1123_date(time.mktime(modified.timetuple()))
+
+        >>> modified = datetime(2007,12,31,tzinfo=pytz.utc)
+        >>> modHeader = zope.datetime.rfc1123_date(zope.datetime.time(modified.isoformat()))
         >>> request = TestRequest(IF_MODIFIED_SINCE=modHeader)
-        
+
         >>> view = FileTestView(aFile,request)
         >>> view.show()
         ''
         >>> request.response.getStatus()
         304
-        
+
         """
 
         if self.request is not None:
@@ -101,14 +101,14 @@ class FileView(object):
             self.request.response.setHeader('Content-Length',
                                             self.context.getSize())
         try:
-            modified = IZopeDublinCore(self.context).modified
+            modified = IDCTimes(self.context).modified
         except TypeError:
             modified=None
         if modified is None or not isinstance(modified,datetime):
             return self.context.data
-        
+
         header= self.request.getHeader('If-Modified-Since', None)
-        lmt = long(time.mktime(modified.timetuple()))
+        lmt = zope.datetime.time(modified.isoformat())
         if header is not None:
             header = header.split(';')[0]
             try:    mod_since=long(zope.datetime.time(header))
@@ -122,6 +122,8 @@ class FileView(object):
 
         return self.context.data
 
+def cleanupFileName(filename):
+    return filename.split('\\')[-1].split('/')[-1]
 
 class FileUpdateView(object):
 
@@ -135,6 +137,7 @@ class FileUpdateView(object):
             filename = getattr(form["field.data"], "filename", None)
             contenttype = form.get("field.contentType")
             if filename:
+                filename = cleanupFileName(filename)
                 if not contenttype:
                     contenttype = guess_content_type(filename)[0]
                 if not form.get("add_input_name"):
@@ -337,8 +340,15 @@ class FileEdit(object):
         >>> view.getData()
         {'data': u'', 'contentType': ''}
 
+        We install an event logger so we can see the events generated.
+
+        >>> def eventLog(event):
+        ...    print event.__class__.__name__, event.descriptions[0].attributes
+        >>> zope.event.subscribers.append(eventLog)
+
         >>> view.setData({'contentType': 'text/plain; charset=ISO-8859-13',
-        ...               'data': u'text \u0105'})
+        ...               'data': u'text \u0105'}) # doctest:+ELLIPSIS
+        ObjectModifiedEvent ('data', 'contentType')
         u'Updated on ${date_time}'
 
         >>> view.context.contentType
@@ -349,6 +359,10 @@ class FileEdit(object):
         >>> view.getData()['data']
         u'text \u0105'
 
+        Cleanup eventlog.
+
+        >>> zope.event.subscribers.remove(eventLog)
+
     You will get an error if you try to specify a charset that cannot encode
     all the characters
 
@@ -431,14 +445,28 @@ class FileEdit(object):
     def setData(self, data):
         charset = extractCharset(data['contentType'])
         try:
-            self.context.data = data['data'].encode(charset)
+            encodeddata = data['data'].encode(charset)
         except LookupError:
             raise UnknownCharset(charset)
         except UnicodeEncodeError:
             raise CharsetTooWeak(charset)
-        self.context.contentType = data['contentType']
+        
+        modified = []
+        if encodeddata != self.context.data:
+            self.context.data = encodeddata
+            modified.append('data')
+        
+        if self.context.contentType != data['contentType']:
+            self.context.contentType = data['contentType']
+            modified.append('contentType')
         formatter = self.request.locale.dates.getFormatter('dateTime',
                                                            'medium')
+        if modified:
+            event = lifecycleevent.ObjectModifiedEvent(
+                self.context,
+                lifecycleevent.Attributes(IFile, *modified))
+            zope.event.notify(event)
+
         return _("Updated on ${date_time}",
                  mapping={'date_time': formatter.format(datetime.utcnow())})
 
@@ -460,16 +488,16 @@ class FileEdit(object):
 def extractCharset(content_type):
     """Extract charset information from a MIME type.
 
-        >>> extractCharset('text/plain; charset=UTF-8')
-        'UTF-8'
+        >>> extractCharset('text/plain; charset=US-ASCII')
+        'US-ASCII'
         >>> extractCharset('text/html; charset=ISO-8859-1')
         'ISO-8859-1'
         >>> extractCharset('text/plain')
-        'ASCII'
+        'UTF-8'
 
     """
     if content_type and content_type.strip():
         major, minor, params = contenttype.parse(content_type)
-        return params.get("charset", "ASCII")
+        return params.get("charset", "UTF-8")
     else:
-        return "ASCII"
+        return "UTF-8"
