@@ -13,7 +13,7 @@
 ##############################################################################
 """Image content type implementation
 
-$Id: image.py 70393 2006-09-27 07:28:49Z dobe $
+$Id: image.py 76693 2007-06-14 13:39:36Z mgedmin $
 """
 __docformat__ = 'restructuredtext'
 
@@ -25,7 +25,7 @@ from zope.size.interfaces import ISized
 from zope.size import byteDisplay
 from zope.contenttype import guess_content_type
 
-from zope.app.i18n import ZopeMessageFactory as _
+from zope.app.file.i18n import ZopeMessageFactory as _
 from zope.app.file.file import File
 from zope.app.file.interfaces import IImage
 
@@ -152,5 +152,11 @@ def getImageInfo(data):
         except ValueError:
             pass
 
+    # handle BMPs
+    elif (size >= 30) and data.startswith('BM'):
+        kind = struct.unpack("<H", data[14:16])[0]
+        if kind == 40: # Windows 3.x bitmap
+            content_type = 'image/x-ms-bmp'
+            width, height = struct.unpack("<LL", data[18:26])
 
     return content_type, width, height
