@@ -13,7 +13,7 @@
 ##############################################################################
 """This module tests the regular persistent Translation Domain.
 
-$Id: test_translationdomain.py 39062 2005-10-11 18:36:32Z philikon $
+$Id: test_translationdomain.py 80989 2007-10-24 00:07:38Z fdrake $
 """
 import unittest
 
@@ -27,15 +27,14 @@ from zope.i18n.translationdomain \
 from zope.interface import implements, classImplements
 from zope.interface.verify import verifyObject
 from zope.testing.doctestunit import DocTestSuite
+from zope.annotation.interfaces import IAttributeAnnotatable
 
 from zope.app import zapi
-from zope.app.annotation.interfaces import IAttributeAnnotatable
-from zope.app.i18n.interfaces import ISyncTranslationDomain
+from zope.app.i18n import interfaces
 from zope.app.i18n.messagecatalog import MessageCatalog
 from zope.app.i18n.translationdomain import TranslationDomain
 from zope.app.component.testing import PlacefulSetup
 from zope.app.testing import setup, ztapi
-from zope.app.component.interfaces import ILocalUtility 
 
 
 class Environment(object):
@@ -143,7 +142,7 @@ class TestISyncTranslationDomain(object):
         self._domain = self._getTranslationDomain()
 
     def testInterface(self):
-        verifyObject(ISyncTranslationDomain, self._domain)
+        verifyObject(interfaces.ISyncTranslationDomain, self._domain)
 
     def testGetMessagesMapping(self):
         mapping = self._domain.getMessagesMapping(['de', 'en'],
@@ -187,7 +186,7 @@ class TestTranslationDomain(TestITranslationDomain,
 
 
     def setUp(self):
-        classImplements(TranslationDomain, ILocalUtility, IAttributeAnnotatable)
+        classImplements(TranslationDomain, IAttributeAnnotatable)
         self.sm = PlacefulSetup.setUp(self, site=True)
         TestITranslationDomain.setUp(self)
 
@@ -221,37 +220,45 @@ class TestTranslationDomain(TestITranslationDomain,
             self._domain.translate('short_greeting', target_language='en'),
             'Hello!')
 
+    def testCatalogDomain(self):
+        domain = self._domain
+        domain.domain = 'myfault'
+        domain.addMessage('greeting2', 'Hola!', 'ca')
+        self.assertEqual(domain['ca'].domain, domain.domain)
+        domain.domain = 'default'
 
 class TestTranslationDomainInAction(unittest.TestCase):
 
     def setUp(self):
         setup.placefulSetUp()
         self.rootFolder = setup.buildSampleFolderTree()
-        sm = zapi.getGlobalSiteManager()
+        gsm = zapi.getGlobalSiteManager()
         de_catalog = MessageCatalog('de', 'default')
         de_catalog.setMessage('short_greeting', 'Hallo!', 10)
-
-        # Create global translation domain and add the catalog.
+        de_catalog.setMessage('long_greeting', 'Guten Tag!', 10)
+        
+        # register global translation domain and add the catalog.
         domain = GlobalTranslationDomain('default')
         domain.addCatalog(de_catalog)
-        sm.provideUtility(ITranslationDomain, domain, 'default')
+        gsm.registerUtility(domain, ITranslationDomain, 'default')
 
-        # Create Domain in root folder
-        mgr = setup.createSiteManager(self.rootFolder)
-        self.trans = setup.addDomain(mgr, Translation, TranslationDomain())
-
-        # Create Domain in folder1
-        mgr = setup.createSiteManager(zapi.traverse(self.rootFolder, 'folder1'))
+        # create a local site manager and add a local translation domain
         td = TranslationDomain()
         td.domain = 'default'
         de_catalog = MessageCatalog('de', 'default')
         de_catalog.setMessage('short_greeting', 'Hallo Welt!', 10)
         td['de-default-1'] = de_catalog
-        self.trans1 = setup.addDomain(mgr, Translation, ts)
 
+        mgr = setup.createSiteManager(
+            zapi.traverse(self.rootFolder, 'folder1'))
+        setup.addUtility(
+            mgr, 'default', interfaces.ILocalTranslationDomain, td)
+
+        self.trans1 = td
+        self.trans = domain
+        
     def tearDown(self):
         setup.placefulTearDown()
-        
 
     def test_translate(self):
         self.assertEqual(
@@ -263,12 +270,16 @@ class TestTranslationDomainInAction(unittest.TestCase):
                                   target_language='de'),
             'Hallo Welt!')
 
-
+        self.assertEqual(
+            self.trans1.translate('long_greeting', 'default',
+                                  target_language='de'),
+            'Guten Tag!')
+        
 def test_suite():
     return unittest.TestSuite((
         unittest.makeSuite(TestTranslationDomain),
         DocTestSuite('zope.app.i18n.translationdomain'),
-        #unittest.makeSuite(TestTranslationDomainInAction),
+        unittest.makeSuite(TestTranslationDomainInAction),
         ))
 
 if __name__=='__main__':
