@@ -13,7 +13,7 @@
 ##############################################################################
 """Define view component for image editing.
 
-$Id: i18nimage.py 67630 2006-04-27 00:54:03Z jim $
+$Id: i18nimage.py 81056 2007-10-24 20:24:02Z srichter $
 """
 __docformat__ = 'restructuredtext'
 
@@ -22,7 +22,7 @@ from urllib import quote
 from zope.i18n.negotiator import negotiator
 from zope.size import ISized
 
-from zope.app.i18n import ZopeMessageFactory as _
+from zope.i18nmessageid import ZopeMessageFactory as _
 from zope.app.file.browser.image import ImageData
 
 
