@@ -13,19 +13,19 @@
 ##############################################################################
 """Role Permission View Classes
 
-$Id: rolepermissionview.py 67630 2006-04-27 00:54:03Z jim $
+$Id: rolepermissionview.py 80149 2007-09-26 22:00:18Z rogerineichen $
 """
 from datetime import datetime
 
 from zope.i18n import translate
 from zope.interface import implements
 from zope.exceptions.interfaces import UserError
+from zope.i18nmessageid import ZopeMessageFactory as _
 
 from zope.app import zapi
-from zope.app.i18n import ZopeMessageFactory as _
-from zope.app.security.settings import Unset, Allow, Deny
 from zope.app.security.interfaces import IPermission
-from zope.app.securitypolicy.interfaces import IRole, IRolePermissionManager
+from zope.securitypolicy.interfaces import Unset, Allow, Deny
+from zope.securitypolicy.interfaces import IRole, IRolePermissionManager
 
 class RolePermissionView(object):
 
