@@ -13,15 +13,26 @@
 ##############################################################################
 """Doc tests for the FTP server.
 
-$Id: test_ftp.py 26201 2004-07-08 10:42:46Z srichter $
+$Id: test_ftp.py 89850 2008-08-15 00:04:15Z ccomb $
 """
 import unittest
-from zope.testing.doctestunit import DocTestSuite
+
+class Tests(unittest.TestCase):
+
+    def test_ftp(self):
+        from ZODB.tests.util import DB
+        from zope.app.server.ftp import FTPRequestFactory
+        from cStringIO import StringIO
+        db = DB()
+        factory = FTPRequestFactory(db)
+        request = factory(StringIO(''), {'credentials': None, 'path': '/'})
+        self.assertTrue(request.publication.db is db)
+        db.close()
+
 
 def test_suite():
-    return unittest.TestSuite((
-        DocTestSuite('zope.app.server.ftp'),
-        ))
+    loader = unittest.TestLoader()
+    return loader.loadTestsFromTestCase(Tests)
 
 if __name__ == '__main__':
     unittest.main(defaultTest='test_suite')
