@@ -12,7 +12,7 @@
 #
 ##############################################################################
 """
-$Id: trialtest.py 66373 2006-04-04 10:02:50Z mkerrin $
+$Id: trialtest.py 72310 2007-02-01 21:39:01Z mkerrin $
 """
 __docformat__="restructuredtext"
 
@@ -33,7 +33,7 @@ class TestTrialTests(twisted.trial.unittest.TestCase):
         pass
 
     def test_error(self):
-        raise Exception, "this test is a broken trial test :-)"
+        raise Exception("this test is a broken trial test :-)")
 
     def test_failure(self):
         self.assert_(False, "I am a failed trial test")
