@@ -12,7 +12,7 @@
 #
 ##############################################################################
 """
-$Id: trialtestfs.py 66373 2006-04-04 10:02:50Z mkerrin $
+$Id: trialtestfs.py 72310 2007-02-01 21:39:01Z mkerrin $
 """
 __docformat__="restructuredtext"
 
@@ -23,13 +23,13 @@ import twisted.trial.unittest
 class TestTrialBrokenSetup(twisted.trial.unittest.TestCase):
 
     def setUp(self):
-        raise Exception, 'broken setup'
+        raise Exception("broken setup")
 
     def tearDown(self):
         pass
 
     def test_error(self):
-        raise Exception, "this test is a broken trial test :-)"
+        raise Exception("this test is a broken trial test :-)")
 
     def test_failure(self):
         self.assert_(False, "I am a failed trial test")
