@@ -12,7 +12,7 @@
 #
 ##############################################################################
 """
-$Id: trialtestft.py 66373 2006-04-04 10:02:50Z mkerrin $
+$Id: trialtestft.py 72310 2007-02-01 21:39:01Z mkerrin $
 """
 __docformat__="restructuredtext"
 
@@ -26,10 +26,10 @@ class TestTrialBrokenTeardown(twisted.trial.unittest.TestCase):
         pass
 
     def tearDown(self):
-        raise Exception, 'broken tearDown'
+        raise Exception("broken tearDown")
 
     def test_error(self):
-        raise Exception, "this test is a broken trial test :-)"
+        raise Exception("this test is a broken trial test :-)")
 
     def test_failure(self):
         self.assert_(False, "I am a failed trial test")
