@@ -13,7 +13,7 @@
 ##############################################################################
 """HTTP server factories
 
-$Id: http.py 41110 2006-01-03 19:01:56Z jim $
+$Id: http.py 72310 2007-02-01 21:39:01Z mkerrin $
 """
 
 from cStringIO import StringIO
@@ -29,11 +29,11 @@ from zope.app.wsgi import PMDBWSGIPublisherApplication
 max_stringio = 100*1000 # Should this be configurable?
 
 class Prebuffer(resource.WrapperResource):
-    def hook(self, ctx):
-        req = iweb.IRequest(ctx)
 
-        content_length = req.headers.getHeader('content-length')
-        if content_length is not None and int(content_length) > max_stringio:
+    def hook(self, req):
+        content_length = req.stream.length
+
+        if content_length > max_stringio:
             temp = tempfile.TemporaryFile()
             def done(_):
                 temp.seek(0)
@@ -41,7 +41,6 @@ class Prebuffer(resource.WrapperResource):
                 req.stream = stream.FileStream(temp, useMMap=False)
                 # Hm, this shouldn't be required:
                 req.stream.doStartReading = None
-
         else:
             temp = StringIO()
             def done(_):
@@ -49,15 +48,9 @@ class Prebuffer(resource.WrapperResource):
                 req.stream = stream.MemoryStream(temp.getvalue())
                 # Hm, this shouldn't be required:
                 req.stream.doStartReading = None
-            
+
         return stream.readStream(req.stream, temp.write).addCallback(done)
 
-    # Oops, fix missing () in lambda in WrapperResource
-    def locateChild(self, ctx, segments):
-        x = self.hook(ctx)
-        if x is not None:
-            return x.addCallback(lambda data: (self.res, segments))
-        return self.res, segments
 
 def createHTTPFactory(db):
     resource = wsgi.WSGIResource(WSGIPublisherApplication(db))
@@ -69,6 +62,7 @@ def createHTTPFactory(db):
 http = ServerType(createHTTPFactory, 8080)
 https = SSLServerType(createHTTPFactory, 8443)
 
+
 def createPMHTTPFactory(db):
     resource = wsgi.WSGIResource(PMDBWSGIPublisherApplication(db))
     resource = log.LogWrapperResource(resource)
