@@ -13,7 +13,7 @@
 ##############################################################################
 """Component Architecture Tests
 
-$Id: test_api.py 28632 2004-12-16 17:42:59Z srichter $
+$Id: tests.py 77896 2007-07-14 08:35:35Z nikhil_n $
 """
 import re
 import unittest
@@ -958,25 +958,46 @@ class StandaloneTests(unittest.TestCase):
         import os
         import StringIO
         import tempfile
+        import pickle
+
         executable = os.path.abspath(sys.executable)
         program = os.path.join(os.path.dirname(__file__), 'standalonetests.py')
-        paths = ':'.join(p for p in sys.path if p)
-        command = "%(executable)s %(program)s %(paths)s" % {
-            'executable': executable, 'program': program, 'paths': paths}
-        t = tempfile.TemporaryFile()
-        res = subprocess.call([executable, program, paths], stdout=t, stderr=t)
-        t.seek(0)
-        lines = t.readlines()
-        t.close()
+        process = subprocess.Popen([executable, program],
+                                   stdout=subprocess.PIPE,
+                                   stderr=subprocess.STDOUT,
+                                   stdin=subprocess.PIPE)
+        pickle.dump(sys.path, process.stdin)
+        process.stdin.close()
+
+        try:
+            process.wait()
+        except OSError, e:
+            if e.errno != 4: # MacIntel raises apparently unimportant EINTR?
+                raise # TODO verify sanity of a pass on EINTR :-/
+        lines = process.stdout.readlines()
+        process.stdout.close()
+        success = True
+        # Interpret the result: We scan the output from the end backwards
+        # until we find either a line that says 'OK' (which means the tests
+        # ran successfully) or a line that starts with quite a few dashes
+        # (which means we didn't find a line that says 'OK' within the summary
+        # of the test runner and the tests did not run successfully.)
         for l in reversed(lines):
             l = l.strip()
-            if l:
-                if not l.endswith('OK'):
-                    self.fail(''.join(lines))
-                else:
-                    break
+            if not l:
+                continue
+            if l.startswith('-----'):
+                break
+            if l.endswith('OK'):
+                sucess = True
+        if not success:
+            self.fail(''.join(lines))
+
+def setUpRegistryTests(tests):
+    setUp()
 
 def tearDownRegistryTests(tests):
+    tearDown()
     import zope.event
     zope.event.subscribers.pop()
 
@@ -990,6 +1011,8 @@ def clearZCML(test=None):
 def test_suite():
     checker = renormalizing.RENormalizing([
         (re.compile('at 0x[0-9a-fA-F]+'), 'at <SOME ADDRESS>'),
+        (re.compile(r"<type 'exceptions.(\w+)Error'>:"),
+                    r'exceptions.\1Error:'),
         ])
 
     return unittest.TestSuite((
@@ -1003,10 +1026,11 @@ def test_suite():
         doctest.DocFileSuite('factory.txt',
                              setUp=setUp, tearDown=tearDown),
         doctest.DocFileSuite('registry.txt', checker=checker,
+                             setUp=setUpRegistryTests,
                              tearDown=tearDownRegistryTests),
         doctest.DocFileSuite('event.txt',
                              setUp=setUp, tearDown=tearDown),
-        doctest.DocFileSuite('zcml.txt',
+        doctest.DocFileSuite('zcml.txt',checker=checker,
                              setUp=setUp, tearDown=tearDown),
         unittest.makeSuite(StandaloneTests),
         ))
