@@ -18,11 +18,14 @@ big improvement of the 'BrowserRequest' to 'HTTPRequest' is that is can handle
 HTML form data and convert them into a Python-native format. Even file data is
 packaged into a nice, Python-friendly 'FileUpload' object.
 
-$Id: browser.py 68702 2006-06-17 05:25:20Z ctheune $
+$Id: browser.py 78536 2007-08-02 00:36:05Z philikon $
 """
+__docformat__ = 'restructuredtext'
+
 import re
 from types import ListType, TupleType, StringType
 from cgi import FieldStorage
+import tempfile
 
 import zope.component
 from zope.interface import implements, directlyProvides
@@ -39,6 +42,7 @@ from zope.publisher.interfaces.browser import IBrowserApplicationRequest
 from zope.publisher.interfaces.browser import IBrowserView
 from zope.publisher.interfaces.browser import IBrowserPage
 from zope.publisher.interfaces.browser import IBrowserSkinType
+from zope.publisher.interfaces.browser import ISkinChangedEvent
 from zope.publisher.interfaces.http import IHTTPRequest
 from zope.publisher.http import HTTPRequest, HTTPResponse
 
@@ -260,7 +264,8 @@ class BrowserRequest(HTTPRequest):
         if 'QUERY_STRING' not in self._environ:
             self._environ['QUERY_STRING'] = ''
 
-        fs = FieldStorage(fp=fp, environ=self._environ, keep_blank_values=1)
+        fs = ZopeFieldStorage(fp=fp, environ=self._environ,
+                              keep_blank_values=1)
 
         fslist = getattr(fs, 'list', None)
         if fslist is not None:
@@ -356,8 +361,10 @@ class BrowserRequest(HTTPRequest):
                 # skip over empty fields
                 return
 
-        # Make it unicode
-        key = self._decode(key)
+        # Make it unicode if not None
+        if key is not None:
+            key = self._decode(key)
+
         if type(item) == StringType:
             item = self._decode(item)
 
@@ -553,17 +560,17 @@ class BrowserRequest(HTTPRequest):
 
     def get(self, key, default=None):
         'See Interface.Common.Mapping.IReadMapping'
+        marker = object()
+        result = self.form.get(key, marker)
+        if result is not marker:
+            return result
 
-        result = self.form.get(key, self)
-        if result is not self: return result
+        return super(BrowserRequest, self).get(key, default)
 
-        result = self._cookies.get(key, self)
-        if result is not self: return result
+class ZopeFieldStorage(FieldStorage):
 
-        result = self._environ.get(key, self)
-        if result is not self: return result
-
-        return default
+    def make_file(self, binary=None):
+        return tempfile.NamedTemporaryFile('w+b')
 
 
 class FileUpload(object):
@@ -586,7 +593,8 @@ class FileUpload(object):
         else:
             methods = ['close', 'fileno', 'flush', 'isatty',
                 'read', 'readline', 'readlines', 'seek',
-                'tell', 'truncate', 'write', 'writelines']
+                'tell', 'truncate', 'write', 'writelines',
+                'name']
 
         d = self.__dict__
         for m in methods:
@@ -607,8 +615,7 @@ class TestRequest(BrowserRequest):
     """
 
     def __init__(self, body_instream=None, environ=None, form=None,
-                 skin=None, outstream=None,
-                 **kw):
+                 skin=None, **kw):
 
         _testEnv =  {
             'SERVER_URL':         'http://127.0.0.1',
@@ -618,18 +625,6 @@ class TestRequest(BrowserRequest):
             }
 
         if environ is not None:
-            # BBB: This is backward-compatibility support for the deprecated
-            # output stream.
-            try:
-                environ.get
-            except AttributeError:
-                import warnings
-                warnings.warn("Can't pass output streams to requests anymore. "
-                              "This will go away in Zope 3.4.",
-                              DeprecationWarning,
-                              2)
-                environ, form, skin, outstream = form, skin, outstream, environ
-
             _testEnv.update(environ)
 
         if kw:
@@ -656,13 +651,6 @@ class TestRequest(BrowserRequest):
         else:
             directlyProvides(self, IDefaultBrowserLayer)
 
-        # BBB: Goes away in 3.4.
-        self.response.outstream = outstream
-
-    # BBB: Remove in 3.4. The super version will be ok.
-    def _createResponse(self):
-        return BBBResponse()
-
 
 
 class BrowserResponse(HTTPResponse):
@@ -744,30 +732,6 @@ class BrowserResponse(HTTPResponse):
         super(BrowserResponse, self).reset()
         self._base = ''
 
-class BBBResponse(BrowserResponse):
-
-    def write(self, str):
-        import warnings
-        warnings.warn("write() method does not exist anymore.",
-                      DeprecationWarning,
-                      2)
-        self.outstream.write(str)
-
-    def outputBody(self):
-        import warnings
-        warnings.warn("Can't pass output streams to requests anymore",
-                      DeprecationWarning,
-                      2)
-        self.outstream.write("Status: %s\r\n" % self.getStatusString())
-        headers = self.getHeaders()
-        headers.sort()
-        self.outstream.write(
-            "\r\n".join([("%s: %s" % h) for h in headers])
-            + "\r\n\r\n"
-            )
-        self.outstream.write(''.join(self.getBody()))
-
-
 def isHTML(str):
      """Try to determine whether str is HTML or not."""
      s = str.lstrip().lower()
@@ -1019,6 +983,22 @@ def applySkin(request, skin):
     >>> pprint.pprint(list(providedBy(req).interfaces()))
     [<InterfaceClass zope.publisher.browser.SkinB>,
      <InterfaceClass zope.publisher.browser.IRequest>]
+
+    Changing the skin on a request triggers the ISkinChanged event:
+
+    >>> import zope.component
+    >>> from zope.publisher.interfaces.browser import ISkinChangedEvent
+    >>> def receiveSkinEvent(event):
+    ...     print event.request
+    >>> zope.component.provideHandler(receiveSkinEvent, (ISkinChangedEvent,))
+    >>> applySkin(req, SkinA)   # doctest: +ELLIPSIS
+    <zope.publisher.browser.Request object at 0x...>
+
+    Make sure our registrations go away again.
+
+    >>> from zope.testing.cleanup import cleanUp
+    >>> cleanUp()
+
     """
     # Remove all existing skin declarations (commonly the default skin).
     ifaces = [iface for iface in directlyProvidedBy(request)
@@ -1026,3 +1006,11 @@ def applySkin(request, skin):
     # Add the new skin.
     ifaces.append(skin)
     directlyProvides(request, *ifaces)
+    zope.event.notify(SkinChangedEvent(request))
+
+class SkinChangedEvent(object):
+
+    zope.interface.implements(ISkinChangedEvent)
+
+    def __init__(self, request):
+        self.request = request
