@@ -13,7 +13,7 @@
 ##############################################################################
 """Schema interfaces and exceptions
 
-$Id: interfaces.py 67630 2006-04-27 00:54:03Z jim $
+$Id: interfaces.py 77624 2007-07-08 14:02:38Z ctheune $
 """
 __docformat__ = "reStructuredText"
 
@@ -324,6 +324,9 @@ class IInt(IMinMax, IField):
 class IFloat(IMinMax, IField):
     u"""Field containing a Float."""
 
+class IDecimal(IMinMax, IField):
+    u"""Field containing a Decimal."""
+
 class IDatetime(IMinMax, IField):
     u"""Field containing a DateTime."""
 
@@ -333,6 +336,9 @@ class IDate(IMinMax, IField):
 class ITimedelta(IMinMax, IField):
     u"""Field containing a timedelta."""
 
+class ITime(IMinMax, IField):
+    u"""Field containing a time."""
+
 def _is_field(value):
     if not IField.providedBy(value):
         return False
@@ -398,31 +404,31 @@ class IUnorderedCollection(ICollection):
 
 class IAbstractSet(IUnorderedCollection):
     u"""An unordered collection of unique values."""
-    
+
     unique = Attribute(u"This ICollection interface attribute must be True")
 
 class IAbstractBag(IUnorderedCollection):
     u"""An unordered collection of values, with no limitations on whether
     members are unique"""
-    
+
     unique = Attribute(u"This ICollection interface attribute must be False")
 
 # Concrete
 
 class ITuple(ISequence):
-    u"""Field containing a value that implements the API of a conventional 
+    u"""Field containing a value that implements the API of a conventional
     Python tuple."""
 
 class IList(ISequence):
-    u"""Field containing a value that implements the API of a conventional 
+    u"""Field containing a value that implements the API of a conventional
     Python list."""
 
 class ISet(IAbstractSet):
-    u"""Field containing a value that implements the API of a conventional 
+    u"""Field containing a value that implements the API of a conventional
     Python standard library sets.Set or a Python 2.4+ set."""
 
 class IFrozenSet(IAbstractSet):
-    u"""Field containing a value that implements the API of a conventional 
+    u"""Field containing a value that implements the API of a conventional
     Python 2.4+ frozenset."""
 
 # (end Collections)
@@ -433,6 +439,25 @@ class IObject(IField):
     schema = Attribute("schema",
         _(u"The Interface that defines the Fields comprising the Object."))
 
+class IBeforeObjectAssignedEvent(Interface):
+    """An object is going to be assigned to an attribute on another object.
+
+    Subscribers to this event can change the object on this event to change
+    what object is going to be assigned. This is useful, e.g. for wrapping 
+    or replacing objects before they get assigned to conform to application
+    policy.
+
+    """
+
+    object = Attribute("The object that is going to be assigned.")
+
+    name = Attribute("The name of the attribute under which the object "
+                     "will be assigned.")
+
+    context = Attribute("The context object where the object will be "
+                        "assigned to.")
+
+
 class IDict(IMinMaxLen, IIterable, IContainer):
     u"""Field containing a conventional dict.
 
@@ -472,7 +497,7 @@ class ITokenizedTerm(ITerm):
 
 class ITitledTokenizedTerm(ITokenizedTerm):
     """A tokenized term that includes a title."""
-    
+
     title = TextLine(title=_(u"Title"))
 
 class ISource(Interface):
@@ -483,7 +508,7 @@ class ISource(Interface):
 
     Sources can be large (even infinite), in which case, they need to
     be queried to find out what their values are.
-    
+
     """
 
     def __contains__(value):
@@ -508,8 +533,8 @@ class ISourceQueriables(Interface):
 
         - A queryable object
 
-          This is an object for which there is a view is provided for
-          searcing for items.
+          This is an object for which there is a view provided for
+          searching for items.
 
         """
 
@@ -517,7 +542,7 @@ class IContextSourceBinder(Interface):
     def __call__(context):
         """Return a context-bound instance that implements ISource.
         """
-    
+
 
 class IBaseVocabulary(ISource):
     """Representation of a vocabulary.
@@ -571,7 +596,7 @@ class IVocabularyTokenized(IVocabulary):
     """Vocabulary that provides support for tokenized representation.
 
     Terms returned from getTerm() and provided by iteration must
-    conform to ITitledTokenizedTerm.
+    conform to ITokenizedTerm.
     """
 
     def getTermByToken(token):
