@@ -13,7 +13,7 @@
 ##############################################################################
 """Tests for the testing framework.
 
-$Id: tests.py 30492 2005-05-24 20:28:53Z jim $
+$Id: tests.py 90403 2008-08-27 07:50:47Z srichter $
 """
 import os
 import sys
@@ -25,7 +25,10 @@ def test_suite():
         doctest.DocTestSuite('zope.testing.renormalizing'),
         doctest.DocFileSuite('formparser.txt'),
         doctest.DocTestSuite('zope.testing.loggingsupport'),
+        doctest.DocTestSuite('zope.testing.server'),
         testrunner.test_suite(),
+        doctest.DocFileSuite('setupstack.txt'),
+        doctest.DocFileSuite('doctest.txt'),
         ))
 
 if __name__ == '__main__':
