@@ -13,8 +13,10 @@
 ##############################################################################
 """URL Namespace Implementations
 
-$Id: namespace.py 68737 2006-06-18 22:49:03Z ctheune $
+$Id: namespace.py 81818 2007-11-13 13:07:08Z jukart $
 """
+__docformat__ = 'restructuredtext'
+
 import re
 
 import zope.component
@@ -28,6 +30,7 @@ from zope.security.proxy import removeSecurityProxy
 from zope.traversing.interfaces import ITraversable, IPathAdapter
 from zope.traversing.interfaces import TraversalError, IContainmentRoot
 
+
 class UnexpectedParameters(TraversalError):
     "Unexpected namespace parameters were provided."
 
@@ -378,6 +381,7 @@ class vh(view):
         traversal_stack = request.getTraversalStack()
         app_names = []
 
+        name = name.encode('utf8')
         if name:
             try:
                 proto, host, port = name.split(":")
@@ -437,7 +441,7 @@ class adapter(SimpleHandler):
           no adapter
 
         Clean up:
-        
+
           >>> from zope.testing.cleanup import cleanUp
           >>> cleanUp()
         """
